@@ -1,37 +1,35 @@
-#############################################################################
+################################################################################
 ##
 ## This file is part of PyTango, a python binding for Tango
-##
+## 
 ## http://www.tango-controls.org/static/PyTango/latest/doc/html/index.html
 ##
-## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
-##
-## This is free software; you can redistribute it and/or modify
+## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
+## 
+## PyTango is free software: you can redistribute it and/or modify
 ## it under the terms of the GNU Lesser General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or
+## the Free Software Foundation, either version 3 of the License, or
 ## (at your option) any later version.
-##
-## This software is distributed in the hope that it will be useful,
+## 
+## PyTango is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
 ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ## GNU Lesser General Public License for more details.
-##
+## 
 ## You should have received a copy of the GNU Lesser General Public License
-## along with this program; if not, see <http://www.gnu.org/licenses/>.
-###########################################################################
+## along with PyTango.  If not, see <http://www.gnu.org/licenses/>.
+##
+################################################################################
 
 """
 This is an internal PyTango module.
 """
 
-__all__ = []
+__all__ = ["callback_init"]
 
 __docformat__ = "restructuredtext"
 
-from _PyTango import CmdDoneEvent, AttrReadEvent, AttrWrittenEvent
-# __CallBackAutoDie, __CallBackPushEvent
-from utils import document_method as __document_method
-
+from ._PyTango import CmdDoneEvent, AttrReadEvent, AttrWrittenEvent
 
 def __init_Callback():
     pass
@@ -76,7 +74,7 @@ def __doc_Callback():
             - ext        :
     """
 
-def init(doc=True):
+def callback_init(doc=True):
     __init_Callback()
     if doc:
         __doc_Callback()
