@@ -25,26 +25,26 @@
 This is an internal PyTango module.
 """
 
-__all__ = [ "DeviceClass" ]
+from __future__ import print_function
+
+__all__ = [ "DeviceClass", "device_class_init" ]
 
 __docformat__ = "restructuredtext"
 
-import sys
-import types
-import operator
+import collections
 
-from _PyTango import Except, DevFailed
-from _PyTango import _DeviceClass, Database
-from _PyTango import CmdArgType, AttrDataFormat, AttrWriteType, DispLevel
-from _PyTango import UserDefaultAttrProp
+from ._PyTango import Except, DevFailed, _DeviceClass, CmdArgType, \
+    DispLevel, UserDefaultAttrProp
+from .pyutil import Util
 
-from pyutil import Util
+from .utils import is_pure_str, is_non_str_seq, seqStr_2_obj, obj_2_str, \
+    is_array
+from .utils import document_method as __document_method
 
-from utils import DbData_2_dict, seqStr_2_obj, obj_2_str, is_array
-from utils import document_method as __document_method
+from .globals import get_class, get_class_by_class, \
+    get_constructed_class_by_class
+from .attr_data import AttrData
 
-from globals import get_classes, get_class, get_class_by_class
-from globals import get_constructed_classes, get_constructed_class, get_constructed_class_by_class
 
 class PropUtil:
     """An internal Property util class"""
@@ -67,7 +67,7 @@ class PropUtil:
     def __init__(self):
         self.db = None
         if Util._UseDb:
-            self.db = Database()
+            self.db = Util.instance().get_database()
 
     def set_default_property_values(self, dev_class, class_prop, dev_prop):
         """
@@ -82,14 +82,14 @@ class PropUtil:
 
             Return     : None
         """
-        for name in class_prop.keys():
+        for name in class_prop:
             type = self.get_property_type(name, class_prop)
             val  = self.get_property_values(name, class_prop)
             val  = self.values2string(val, type)
             desc = self.get_property_description(name, class_prop)
             dev_class.add_wiz_class_prop(name, desc, val)
 
-        for name in dev_prop.keys():
+        for name in dev_prop:
             type = self.get_property_type(name, dev_prop)
             val  = self.get_property_values(name, dev_prop)
             val  = self.values2string(val, type)
@@ -113,16 +113,16 @@ class PropUtil:
             return
 
         # call database to get properties
-        props = self.db.get_class_property(dev_class.get_name(), class_prop.keys())
+        props = self.db.get_class_property(dev_class.get_name(), list(class_prop.keys()))
 
         # if value defined in database, store it
-        for name in class_prop.keys():
+        for name in class_prop:
             if props[name]:
                 type   = self.get_property_type(name, class_prop)
                 values = self.stringArray2values(props[name], type)
                 self.set_property_values(name, class_prop, values)
             else:
-                print name, " property NOT found in database"
+                print(name + " property NOT found in database")
 
     def get_device_properties(self, dev, class_prop, dev_prop):
         """
@@ -141,9 +141,9 @@ class PropUtil:
             return
 
         #    Call database to get properties
-        props = self.db.get_device_property(dev.get_name(),dev_prop.keys())
+        props = self.db.get_device_property(dev.get_name(), list(dev_prop.keys()))
         #    if value defined in database, store it
-        for name in dev_prop.keys():
+        for name in dev_prop:
             prop_value = props[name]
             if len(prop_value):
                 data_type = self.get_property_type(name, dev_prop)
@@ -181,7 +181,7 @@ class PropUtil:
                     - v : (object) the object to be analysed
 
                 Return     : (bool) True if the object is a sequence or False otherwise"""
-        return operator.isSequenceType(v)
+        return isinstance(v, collections.Sequence)
 
     def is_empty_seq(self, v):
         """
@@ -244,10 +244,10 @@ class PropUtil:
         except:
             val = []
 
-        if is_array(tg_type) or (operator.isSequenceType(val) and not len(val)):
+        if is_array(tg_type) or (isinstance(val, collections.Sequence) and not len(val)):
             return val
         else:
-            if operator.isSequenceType(val) and not type(val) in types.StringTypes:
+            if is_non_str_seq(val):
                 return val[0]
             else:
                 return val
@@ -272,6 +272,7 @@ class PropUtil:
         """internal helper method"""
         return obj_2_str(argin, argout_type)
 
+
 class DeviceClass(_DeviceClass):
     """Base class for all TANGO device-class class.
        A TANGO device-class class is a class where is stored all
@@ -291,12 +292,14 @@ class DeviceClass(_DeviceClass):
             pu.set_default_property_values(self, self.class_property_list,
                                            self.device_property_list)
             pu.get_class_properties(self, self.class_property_list)
-            for prop_name in self.class_property_list.keys():
-                setattr(self, prop_name, pu.get_property_values(prop_name, self.class_property_list))
-        except DevFailed, df:
+            for prop_name in self.class_property_list:
+                if not hasattr(self, prop_name):
+                    setattr(self, prop_name, pu.get_property_values(prop_name,
+                            self.class_property_list))
+        except DevFailed as df:
             print("PyDS: %s: A Tango error occured in the constructor:" % name)
             Except.print_exception(df)
-        except Exception, e:
+        except Exception as e:
             print("PyDS: %s: An error occured in the constructor:" % name)
             print(str(e))
             
@@ -317,158 +320,26 @@ class DeviceClass(_DeviceClass):
     def __attribute_factory(self, attr_list):
         """for internal usage only"""
 
-        for attr_name, attr_info in self.attr_list.iteritems():
-            self.__create_attribute(attr_list, attr_name, attr_info)
+        for attr_name, attr_info in self.attr_list.items():
+            attr_data = AttrData(attr_name, self.get_name(), attr_info)
+            self.__create_attribute(attr_list, attr_data)
 
-    def __create_attribute(self, attr_list, attr_name, attr_info):
+    def __create_attribute(self, attr_list, attr_data):
         """for internal usage only"""
-        name = self.get_name()
-
-        # check for well defined attribute info
-
-        # check parameter
-        if not operator.isSequenceType(attr_info):
-            msg = "Wrong data type for value for describing attribute %s in " \
-                  "class %s\nMust be a sequence with 1 or 2 elements" % (attr_name, name)
-            self.__throw_create_attribute_exception(msg)
-
-        if len(attr_info) < 1 or len(attr_info) > 2:
-            msg = "Wrong number of argument for describing attribute %s in " \
-                  "class %s\nMust be a sequence with 1 or 2 elements" % (attr_name, name)
-            self.__throw_create_attribute_exception(msg)
-
-        extra_info = {}
-        if len(attr_info) == 2:
-            # attr_info[1] must be a dictionary
-            # extra_info = attr_info[1], with all the keys lowercase
-            for k, v in attr_info[1].iteritems():
-                extra_info[k.lower()] = v
-
-        attr_info = attr_info[0]
-
-        attr_info_len = len(attr_info)
-        # check parameter
-        if not operator.isSequenceType(attr_info) or attr_info_len < 3 or attr_info_len > 5:
-            msg = "Wrong data type for describing mandatory information for attribute %s " \
-                  "in class %s\nMust be a sequence with 3, 4 or 5 elements" % (attr_name, name)
-            self.__throw_create_attribute_exception(msg)
-
-        # get data type
-        attr_type = CmdArgType.DevVoid
-        try:
-            attr_type = CmdArgType(attr_info[0])
-        except:
-            msg = "Wrong data type in attribute argument for attribute %s in " \
-                  "class %s\nAttribute data type (first element in first " \
-                  "sequence) must be a PyTango.CmdArgType"
-            self.__throw_create_attribute_exception(msg)
-
-        # get format
-        attr_format = AttrDataFormat.SCALAR
-        try:
-            attr_format = AttrDataFormat(attr_info[1])
-        except:
-            msg = "Wrong data format in attribute argument for attribute %s in " \
-                  "class %s\nAttribute data format (second element in first " \
-                  "sequence) must be a PyTango.AttrDataFormat"
-            self.__throw_create_attribute_exception(msg)
-
-        dim_x, dim_y = 1, 0
-        if attr_format == AttrDataFormat.SCALAR:
-            if attr_info_len != 3:
-                msg = "Wrong data type in attribute argument for attribute %s in " \
-                      "class %s\nSequence describing mandatory attribute parameters " \
-                      "for scalar attribute must have 3 elements"
-                self.__throw_create_attribute_exception(msg)
-        elif attr_format == AttrDataFormat.SPECTRUM:
-            if attr_info_len != 4:
-                msg = "Wrong data type in attribute argument for attribute %s in " \
-                      "class %s\nSequence describing mandatory attribute parameters " \
-                      "for spectrum attribute must have 4 elements"
-                self.__throw_create_attribute_exception(msg)
-            try:
-                dim_x = int(attr_info[3])
-            except:
-                msg = "Wrong data type in attribute argument for attribute %s in " \
-                      "class %s\n4th element in sequence describing mandatory dim_x " \
-                      "attribute parameter for spectrum attribute must be an integer"
-                self.__throw_create_attribute_exception(msg)
-        elif attr_format == AttrDataFormat.IMAGE:
-            if attr_info_len != 5:
-                msg = "Wrong data type in attribute argument for attribute %s in " \
-                      "class %s\nSequence describing mandatory attribute parameters " \
-                      "for image attribute must have 5 elements"
-                self.__throw_create_attribute_exception(msg)
-            try:
-                dim_x = int(attr_info[3])
-            except:
-                msg = "Wrong data type in attribute argument for attribute %s in " \
-                      "class %s\n4th element in sequence describing mandatory dim_x " \
-                      "attribute parameter for image attribute must be an integer"
-                self.__throw_create_attribute_exception(msg)
-            try:
-                dim_y = int(attr_info[4])
-            except:
-                msg = "Wrong data type in attribute argument for attribute %s in " \
-                      "class %s\n5th element in sequence describing mandatory dim_y " \
-                      "attribute parameter for image attribute must be an integer"
-                self.__throw_create_attribute_exception(msg)
-
-        #get write type
-        attr_write = AttrWriteType.READ
-        try:
-            attr_write = AttrWriteType(attr_info[2])
-        except:
-            msg = "Wrong data write type in attribute argument for attribute %s in " \
-                  "class %s\nAttribute write type (third element in first " \
-                  "sequence) must be a PyTango.AttrWriteType"
-            self.__throw_create_attribute_exception(msg)
-
-        # check that the method(s) to be executed exists
-        read_method_name = "read_%s" % attr_name
-        write_method_name = "write_%s" % attr_name
-        is_allowed_name = "is_%s_allowed" % attr_name
-
-        try:
-            display_level = DispLevel(extra_info.get("display level", DispLevel.OPERATOR))
-        except:
-            msg = "Wrong display level in attribute information for attribute %s in " \
-                  "class %s\nAttribute information for display level is not a " \
-                  "PyTango.DispLevel" % (attr_name, name)
-            self.__throw_create_attribute_exception(msg)
-
-        try:
-            polling_period = int(extra_info.get("polling period", -1))
-        except:
-            msg = "Wrong polling period in attribute information for attribute %s in " \
-                  "class %s\nAttribute information for polling period is not an " \
-                  "integer" % (attr_name, name)
-            self.__throw_create_attribute_exception(msg)
-
-        memorized, hw_memorized = extra_info.get("memorized", "false"), False
-        if memorized == "true":
-            memorized, hw_memorized = True, True
-        elif memorized == "true_without_hard_applied":
-            memorized = True
-        else:
-            memorized = False
-
-        att_prop = None
-        if extra_info:
-            att_prop = self.__create_user_default_attr_prop(attr_name, extra_info)
-
-        self._create_attribute(attr_list, attr_name, attr_type, attr_format,
-                               attr_write, dim_x, dim_y,
-                               display_level, polling_period,
-                               memorized, hw_memorized,
-                               read_method_name, write_method_name,
-                               is_allowed_name,
-                               att_prop)
+        self._create_attribute(attr_list, attr_data.attr_name,
+                               attr_data.attr_type, attr_data.attr_format,
+                               attr_data.attr_write, attr_data.dim_x,
+                               attr_data.dim_y, attr_data.display_level,
+                               attr_data.polling_period, attr_data.memorized,
+                               attr_data.hw_memorized,
+                               attr_data.read_method_name,
+                               attr_data.write_method_name,
+                               attr_data.is_allowed_name, attr_data.att_prop)
 
     def __create_user_default_attr_prop(self, attr_name, extra_info):
         """for internal usage only"""
         p = UserDefaultAttrProp()
-        for k, v in extra_info.iteritems():
+        for k, v in extra_info.items():
             k_lower = k.lower()
             method_name = "set_%s" % k_lower.replace(' ','_')
             if hasattr(p, method_name):
@@ -495,7 +366,7 @@ class DeviceClass(_DeviceClass):
                   "The init_device() method does not exist!" % name
             Except.throw_exception("PyDs_WrongCommandDefinition", msg, "command_factory()")
 
-        for cmd_name, cmd_info in self.cmd_list.iteritems():
+        for cmd_name, cmd_info in self.cmd_list.items():
             self.__create_command(deviceimpl_class, cmd_name, cmd_info)
 
     def __create_command(self, deviceimpl_class, cmd_name, cmd_info):
@@ -505,7 +376,7 @@ class DeviceClass(_DeviceClass):
         # check for well defined command info
 
         # check parameter
-        if not operator.isSequenceType(cmd_info):
+        if not isinstance(cmd_info, collections.Sequence):
             msg = "Wrong data type for value for describing command %s in " \
                   "class %s\nMust be a sequence with 2 or 3 elements" % (cmd_name, name)
             self.__throw_create_command_exception(msg)
@@ -517,7 +388,7 @@ class DeviceClass(_DeviceClass):
 
         param_info, result_info = cmd_info[0], cmd_info[1]
 
-        if not operator.isSequenceType(param_info):
+        if not isinstance(param_info, collections.Sequence):
             msg = "Wrong data type in command argument for command %s in " \
                   "class %s\nCommand parameter (first element) must be a sequence" % (cmd_name, name)
             self.__throw_create_command_exception(msg)
@@ -540,14 +411,14 @@ class DeviceClass(_DeviceClass):
         param_desc = ""
         if len(param_info) > 1:
             param_desc = param_info[1]
-            if not type(param_desc) in types.StringTypes:
+            if not is_pure_str(param_desc):
                 msg = "Wrong data type in command parameter for command %s in " \
                       "class %s\nCommand parameter description (second element " \
                       "in first sequence), when given, must be a string"
                 self.__throw_create_command_exception(msg)
 
         # Check result
-        if not operator.isSequenceType(result_info):
+        if not isinstance(result_info, collections.Sequence):
             msg = "Wrong data type in command result for command %s in " \
                   "class %s\nCommand result (second element) must be a sequence" % (cmd_name, name)
             self.__throw_create_command_exception(msg)
@@ -570,7 +441,7 @@ class DeviceClass(_DeviceClass):
         result_desc = ""
         if len(result_info) > 1:
             result_desc = result_info[1]
-            if not type(result_desc) in types.StringTypes:
+            if not is_pure_str(result_desc):
                 msg = "Wrong data type in command result for command %s in " \
                       "class %s\nCommand parameter description (second element " \
                       "in second sequence), when given, must be a string" % (cmd_name, name)
@@ -581,7 +452,7 @@ class DeviceClass(_DeviceClass):
 
         if len(cmd_info) == 3:
             extra_info = cmd_info[2]
-            if not operator.isMappingType(extra_info):
+            if not isinstance(extra_info, collections.Mapping):
                 msg = "Wrong data type in command information for command %s in " \
                       "class %s\nCommand information (third element in sequence), " \
                       "when given, must be a dictionary" % (cmd_name, name)
@@ -593,7 +464,7 @@ class DeviceClass(_DeviceClass):
                       "three elements" % (cmd_name, name)
                 self.__throw_create_command_exception(msg)
 
-            for info_name, info_value in extra_info.iteritems():
+            for info_name, info_value in extra_info.items():
                 info_name_lower = info_name.lower()
                 if info_name_lower == "display level":
                     try:
@@ -604,7 +475,7 @@ class DeviceClass(_DeviceClass):
                               "PyTango.DispLevel" % (cmd_name, name)
                         self.__throw_create_command_exception(msg)
                 elif info_name_lower == "default command":
-                    if not type(info_value) in types.StringTypes:
+                    if not is_pure_str(info_value):
                         msg = "Wrong data type in command information for command %s in " \
                               "class %s\nCommand information for default command is not a " \
                               "string" % (cmd_name, name)
@@ -628,7 +499,7 @@ class DeviceClass(_DeviceClass):
         # check that the method to be executed exists
         try:
             cmd = getattr(deviceimpl_class, cmd_name)
-            if not callable(cmd):
+            if not isinstance(cmd, collections.Callable):
                 msg = "Wrong definition of command %s in " \
                       "class %s\nThe object exists in class but is not " \
                       "a method!" % (cmd_name, name)
@@ -641,7 +512,7 @@ class DeviceClass(_DeviceClass):
         is_allowed_name = "is_%s_allowed" % cmd_name
         try:
             is_allowed = getattr(deviceimpl_class, is_allowed_name)
-            if not callable(is_allowed):
+            if not isinstance(is_allowed, collections.Callable):
                 msg = "Wrong definition of command %s in " \
                       "class %s\nThe object '%s' exists in class but is " \
                       "not a method!" % (cmd_name, name, is_allowed_name)
@@ -677,15 +548,6 @@ class DeviceClass(_DeviceClass):
             tmp_dev_list.append(device)
 
         self.dyn_attr(tmp_dev_list)
-#        try:
-#            self.dyn_attr(tmp_dev_list)
-#        except DevFailed, df:
-#            print("PyDS: %s: A Tango error occured trying to create dynamic attributes" % name)
-#            Except.print_exception(df)
-#        except Exception, e:
-#            print("PyDS: %s: An error occured trying to create dynamic attributes" % klass_name)
-#            print(str(e))
-#            sys.exit()
 
         for dev in tmp_dev_list:
             if Util._UseDb and not Util._FileDb:
@@ -775,7 +637,22 @@ class DeviceClass(_DeviceClass):
                 return
         err_mess = "Device " + name + " not in Tango class device list!"
         Except.throw_exception("PyAPI_CantDestroyDevice",err_mess,"DeviceClass.device_destroyer")
+    
+    def device_name_factory(self, dev_name_list):
+        """
+            device_name_factory(self, dev_name_list) ->  None
+                
+                Create device(s) name list (for no database device server).
+                This method can be re-defined in DeviceClass sub-class for
+                device server started without database. Its rule is to
+                initialise class device name. The default method does nothing.
+            
+            Parameters :
+                - dev_name_list : (seq<str>) sequence of devices to be filled
 
+            Return     : None"""
+        pass
+    
 def __init_DeviceClass():
     pass
 
@@ -797,13 +674,24 @@ def __doc_DeviceClass():
 
     document_method("register_signal", """
     register_signal(self, signo) -> None
+    register_signal(self, signo, own_handler=false) -> None
 
             Register a signal.
             Register this class as class to be informed when signal signo
-            is sent to to the device server process
+            is sent to to the device server process.
+            The second version of the method is available only under Linux.
 
         Parameters :
             - signo : (int) signal identifier
+            - own_handler : (bool) true if you want the device signal handler
+                            to be executed in its own handler instead of being
+                            executed by the signal thread. If this parameter 
+                            is set to true, care should be taken on how the
+                            handler is written. A default false value is provided
+        
+        Throws PyTango.DevFailed:
+            - if the signal number is out of range
+            - if the operating system failed to register a signal for the process.
 
         Return     : None
     """ )
@@ -897,6 +785,29 @@ def __doc_DeviceClass():
         Parameters : None
         Return     : (sequence<PyTango.DeviceImpl>) list of PyTango.DeviceImpl objects for this class
     """ )
+
+    document_method("get_command_list", """
+    get_command_list(self) -> sequence<PyTango.Command>
+
+            Gets the list of PyTango.Command objects for this class
+
+        Parameters : None
+        Return     : (sequence<PyTango.Command>) list of PyTango.Command objects for this class
+        
+        New in PyTango 8.0.0
+    """ )
+
+    document_method("get_cmd_by_name", """
+    get_cmd_by_name(self, (str)cmd_name) -> PyTango.Command
+
+            Get a reference to a command object.
+
+        Parameters :
+            - cmd_name : (str) command name
+        Return     : (PyTango.Command) PyTango.Command object
+        
+        New in PyTango 8.0.0
+    """ )
     
     document_method("add_wiz_dev_prop", """
     add_wiz_dev_prop(self, str, str) -> None
@@ -918,7 +829,7 @@ def __doc_DeviceClass():
         Return     : None
     """ )
 
-def init(doc=True):
+def device_class_init(doc=True):
     __init_DeviceClass()
     if doc:
         __doc_DeviceClass()
