@@ -1,50 +1,57 @@
-#############################################################################
+################################################################################
 ##
 ## This file is part of PyTango, a python binding for Tango
-##
+## 
 ## http://www.tango-controls.org/static/PyTango/latest/doc/html/index.html
 ##
-## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
-##
-## This is free software; you can redistribute it and/or modify
+## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
+## 
+## PyTango is free software: you can redistribute it and/or modify
 ## it under the terms of the GNU Lesser General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or
+## the Free Software Foundation, either version 3 of the License, or
 ## (at your option) any later version.
-##
-## This software is distributed in the hope that it will be useful,
+## 
+## PyTango is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
 ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ## GNU Lesser General Public License for more details.
-##
+## 
 ## You should have received a copy of the GNU Lesser General Public License
-## along with this program; if not, see <http://www.gnu.org/licenses/>.
-###########################################################################
+## along with PyTango.  If not, see <http://www.gnu.org/licenses/>.
+##
+################################################################################
 
 """
 This is an internal PyTango module.
 """
 
-__all__ = []
-            
+from __future__ import with_statement
+
+__all__ = ["device_proxy_init"]
+
 __docformat__ = "restructuredtext"
 
-import sys
-import operator
-import types
 import threading
-import traceback
 
-from _PyTango import StdStringVector
-from _PyTango import DbData, DbDatum
-from _PyTango import AttributeInfo, AttributeInfoEx
-from _PyTango import AttributeInfoList, AttributeInfoListEx
-from _PyTango import DeviceProxy, DeviceAttribute, DeviceData
-from _PyTango import __CallBackAutoDie, __CallBackPushEvent, EventType
-from _PyTango import DevFailed
-from _PyTango import ExtractAs
-from PyTango.utils import seq_2_StdStringVector, StdStringVector_2_seq
-from PyTango.utils import seq_2_DbData, DbData_2_dict
-from utils import document_method as __document_method
+from ._PyTango import StdStringVector, DbData, DbDatum, AttributeInfo, \
+    AttributeInfoEx, AttributeInfoList, AttributeInfoListEx, DeviceProxy, \
+    __CallBackAutoDie, __CallBackPushEvent, EventType, DevFailed, Except, \
+    ExtractAs
+from .utils import is_pure_str, is_non_str_seq, is_integer, \
+    seq_2_StdStringVector, StdStringVector_2_seq, seq_2_DbData, DbData_2_dict
+from .utils import document_method as __document_method
+import collections
+import numbers
+
+class __TangoInfo(object):
+    """Helper class for when DeviceProxy.info() is not available"""
+    
+    def __init__(self):
+        self.dev_class = self.dev_type = 'Device'
+        self.doc_url = 'http://www.esrf.fr/computing/cs/tango/tango_doc/ds_doc/'
+        self.server_host = 'Unknown'
+        self.server_id = 'Unknown'
+        self.server_version = 1
 
 #-------------------------------------------------------------------------------
 # Pythonic API: transform tango commands into methods and tango attributes into
@@ -57,7 +64,12 @@ def __check_read_attribute(dev_attr):
     return dev_attr
 
 def __DeviceProxy__refresh_cmd_cache(self):
-    self.__cmd_cache = [cmd.cmd_name.lower() for cmd in self.command_list_query()]
+    cmd_list = self.command_list_query()
+    cmd_cache = {}
+    for cmd in cmd_list:
+        n = cmd.cmd_name.lower()
+        cmd_cache[n] = cmd, None
+    self.__dict__['__cmd_cache'] = cmd_cache
 
 def __DeviceProxy__refresh_attr_cache(self):
     attr_cache = [attr_name.lower() for attr_name in self.get_attribute_list()]
@@ -68,28 +80,40 @@ def __DeviceProxy__getattr(self, name):
     # ticket http://ipython.scipy.org/ipython/ipython/ticket/229 someday
     # and the ugly trait_names could be removed.
     if name[:2] == "__" or name == 'trait_names':
-        raise AttributeError, name
+        raise AttributeError(name)
     
-    find_cmd = True
-    if not hasattr(self, '__cmd_cache') or name.lower() not in self.__cmd_cache:
+    name_l = name.lower()
+    cmd_info = None
+    if not hasattr(self, '__cmd_cache'):
         try:
             self.__refresh_cmd_cache()
         except:
-            find_cmd = False
-    
-    if find_cmd and name.lower() in self.__cmd_cache:
-        def f(*args,**kwds): return self.command_inout(name, *args, **kwds)
+            pass
+    try:
+        cmd_info = self.__cmd_cache[name_l]
+    except:
+        pass
+        
+    if cmd_info is not None:
+        d, f = cmd_info
+        if f is None:
+            doc =  "%s(%s) -> %s\n\n" % (d.cmd_name, d.in_type, d.out_type)
+            doc += " -  in (%s): %s\n" % (d.in_type, d.in_type_desc)
+            doc += " - out (%s): %s\n" % (d.out_type, d.out_type_desc)
+            def f(*args,**kwds): return self.command_inout(name, *args, **kwds)
+            f.__doc__ = doc
+            self.__cmd_cache[name_l] = d, f
         return f
     
     find_attr = True
-    if not hasattr(self, '__attr_cache') or name.lower() not in self.__attr_cache:
+    if not hasattr(self, '__attr_cache') or name_l not in self.__attr_cache:
         try:
             self.__refresh_attr_cache()
         except:
             find_attr = False
     
-    if not find_attr or name.lower() not in self.__attr_cache:
-        raise AttributeError, name
+    if not find_attr or name_l not in self.__attr_cache:
+        raise AttributeError(name)
     
     return self.read_attribute(name).value
 
@@ -112,16 +136,13 @@ def __DeviceProxy__getAttributeNames(self):
     try:
         lst = [cmd.cmd_name for cmd in self.command_list_query()]
         lst += self.get_attribute_list()
-        lst += map(str.lower, lst)
+        lst += list(map(str.lower, lst))
         lst.sort()
         return lst
     except Exception:
         pass
     return []
 
-def __DeviceProxy__del(self):
-    self.__unsubscribe_event_all()
-
 def __DeviceProxy__getitem(self, key):
     return self.read_attribute(key)
 
@@ -170,7 +191,7 @@ def __DeviceProxy__read_attributes_asynch(self, attr_names, cb=None, extract_as=
         return self.__read_attributes_asynch(attr_names)
 
     cb2 = __CallBackAutoDie()
-    if callable(cb):
+    if isinstance(cb, collections.Callable):
         cb2.attr_read = cb
     else:
         cb2.attr_read = cb.attr_read
@@ -232,7 +253,7 @@ def __DeviceProxy__write_attributes_asynch(self, attr_values, cb=None):
         return self.__write_attributes_asynch(attr_values)
 
     cb2 = __CallBackAutoDie()
-    if callable(cb):
+    if isinstance(cb, collections.Callable):
         cb2.attr_write = cb
     else:
         cb2.attr_write = cb.attr_write
@@ -285,7 +306,7 @@ def __DeviceProxy__get_property(self, propname, value=None):
                      DevFailed from database device
     """
 
-    if type(propname) in types.StringTypes or isinstance(propname, StdStringVector):
+    if is_pure_str(propname) or isinstance(propname, StdStringVector):
         new_value = value
         if new_value is None:
             new_value = DbData()
@@ -296,12 +317,12 @@ def __DeviceProxy__get_property(self, propname, value=None):
         new_value.append(propname)
         self._get_property(new_value)
         return DbData_2_dict(new_value)
-    elif operator.isSequenceType(propname):
+    elif isinstance(propname, collections.Sequence):
         if isinstance(propname, DbData):
             self._get_property(propname)
             return DbData_2_dict(propname)
 
-        if type(propname[0]) in types.StringTypes:
+        if is_pure_str(propname[0]):
             new_propname = StdStringVector()
             for i in propname: new_propname.append(i)
             new_value = value
@@ -348,16 +369,16 @@ def __DeviceProxy__put_property(self, value):
         new_value = DbData()
         new_value.append(value)
         value = new_value
-    elif operator.isSequenceType(value) and not type(value) in types.StringTypes:
+    elif is_non_str_seq(value):
         new_value = seq_2_DbData(value)
-    elif operator.isMappingType(value):
+    elif isinstance(value, collections.Mapping):
         new_value = DbData()
-        for k, v in value.iteritems():
+        for k, v in value.items():
             if isinstance(v, DbDatum):
                 new_value.append(v)
                 continue
             db_datum = DbDatum(k)
-            if operator.isSequenceType(v) and not type(v) in types.StringTypes:
+            if is_non_str_seq(v):
                 seq_2_StdStringVector(v, db_datum.value_string)
             else:
                 db_datum.value_string.append(str(v))
@@ -374,46 +395,47 @@ def __DeviceProxy__delete_property(self, value):
     
             Delete a the given of properties for this device.
             This method accepts the following types as value parameter:
-            1. string [in] - single property to be deleted
-            2. PyTango.DbDatum [in] - single property data to be deleted
-            3. PyTango.DbData [in] - several property data to be deleted
-            4. sequence<string> [in]- several property data to be deleted
-            5. sequence<DbDatum> [in] - several property data to be deleted
-            6. dict<str, obj> [in] - keys are property names to be deleted (values are ignored)
-            7. dict<str, DbDatum> [in] - several DbDatum.name are property names to be
-               deleted (keys are ignored)
+            
+                1. string [in] - single property to be deleted
+                2. PyTango.DbDatum [in] - single property data to be deleted
+                3. PyTango.DbData [in] - several property data to be deleted
+                4. sequence<string> [in]- several property data to be deleted
+                5. sequence<DbDatum> [in] - several property data to be deleted
+                6. dict<str, obj> [in] - keys are property names to be deleted (values are ignored)
+                7. dict<str, DbDatum> [in] - several DbDatum.name are property names to be deleted (keys are ignored)
 
         Parameters :
             - value : can be one of the following:
+            
                 1. string [in] - single property data to be deleted
                 2. PyTango.DbDatum [in] - single property data to be deleted
                 3. PyTango.DbData [in] - several property data to be deleted
                 4. sequence<string> [in]- several property data to be deleted
                 5. sequence<DbDatum> [in] - several property data to be deleted
                 6. dict<str, obj> [in] - keys are property names to be deleted (values are ignored)
-                7. dict<str, DbDatum> [in] - several DbDatum.name are property names
-                   to be deleted (keys are ignored)
+                7. dict<str, DbDatum> [in] - several DbDatum.name are property names to be deleted (keys are ignored)
+        
         Return     : None
 
         Throws     : ConnectionFailed, CommunicationFailed
                     DevFailed from device (DB_SQLError)
     """
     if isinstance(value, DbData) or isinstance(value, StdStringVector) or \
-       type(value) in types.StringTypes:
+       is_pure_str(value):
         new_value = value
     elif isinstance(value, DbDatum):
         new_value = DbData()
         new_value.append(value)
-    elif operator.isSequenceType(value):
+    elif isinstance(value, collections.Sequence):
         new_value = DbData()
         for e in value:
             if isinstance(e, DbDatum):
                 new_value.append(e)
             else:
                 new_value.append(DbDatum(str(e)))
-    elif operator.isMappingType(value):
+    elif isinstance(value, collections.Mapping):
         new_value = DbData()
-        for k, v in value.iteritems():
+        for k, v in value.items():
             if isinstance(v, DbDatum):
                 new_value.append(v)
             else:
@@ -458,7 +480,7 @@ def __DeviceProxy__get_property_list(self, filter, array=None):
     if isinstance(array, StdStringVector):
         self._get_property_list(filter, array)
         return array
-    elif operator.isSequenceType(array):
+    elif isinstance(array, collections.Sequence):
         new_array = StdStringVector()
         self._get_property_list(filter, new_array)
         StdStringVector_2_seq(new_array, array)
@@ -497,9 +519,9 @@ def __DeviceProxy__get_attribute_config(self, value):
 
         Deprecated: use get_attribute_config_ex instead
     """
-    if isinstance(value, StdStringVector) or type(value) in types.StringTypes:
+    if isinstance(value, StdStringVector) or is_pure_str(value):
         return self._get_attribute_config(value)
-    elif operator.isSequenceType(value):
+    elif isinstance(value, collections.Sequence):
         v = seq_2_StdStringVector(value)
         return self._get_attribute_config(v)
 
@@ -535,11 +557,11 @@ def __DeviceProxy__get_attribute_config_ex(self, value):
     """
     if isinstance(value, StdStringVector):
         return self._get_attribute_config_ex(value)
-    elif type(value) in types.StringTypes:
+    elif is_pure_str(value):
         v = StdStringVector()
         v.append(value)
         return self._get_attribute_config_ex(v)
-    elif operator.isSequenceType(value):
+    elif isinstance(value, collections.Sequence):
         v = seq_2_StdStringVector(value)
         return self._get_attribute_config_ex(v)
 
@@ -603,7 +625,7 @@ def __DeviceProxy__set_attribute_config(self, value):
         v = value
     elif isinstance(value, AttributeInfoListEx):
         v = value
-    elif operator.isSequenceType(value):
+    elif isinstance(value, collections.Sequence):
         if not len(value): return
         if isinstance(value[0], AttributeInfoEx):
             v = AttributeInfoListEx()
@@ -619,6 +641,15 @@ def __DeviceProxy__set_attribute_config(self, value):
 
     return self._set_attribute_config(v)
 
+def __DeviceProxy__get_event_map_lock(self):
+    """
+    Internal helper method"""
+    if not hasattr(self, '_subscribed_events_lock'):
+        # do it like this instead of self._subscribed_events = dict() to avoid
+        # calling __setattr__ which requests list of tango attributes from device
+        self.__dict__['_subscribed_events_lock'] = threading.Lock()
+    return self._subscribed_events_lock
+
 def __DeviceProxy__get_event_map(self):
     """
     Internal helper method"""
@@ -693,13 +724,13 @@ def __DeviceProxy__subscribe_event ( self, attr_name, event_type, cb_or_queuesiz
             other subscribe_event() version.
     """
     
-    if callable(cb_or_queuesize):
+    if isinstance(cb_or_queuesize, collections.Callable):
         cb = __CallBackPushEvent()
         cb.push_event = cb_or_queuesize
-    elif hasattr(cb_or_queuesize, "push_event") and callable(cb_or_queuesize.push_event):
+    elif hasattr(cb_or_queuesize, "push_event") and isinstance(cb_or_queuesize.push_event, collections.Callable):
         cb = __CallBackPushEvent()
         cb.push_event = cb_or_queuesize.push_event
-    elif operator.isNumberType(cb_or_queuesize):
+    elif is_integer(cb_or_queuesize):
         cb = cb_or_queuesize # queuesize
     else:
         raise TypeError("Parameter cb_or_queuesize should be a number, a" + \
@@ -707,8 +738,16 @@ def __DeviceProxy__subscribe_event ( self, attr_name, event_type, cb_or_queuesiz
 
     event_id = self.__subscribe_event(attr_name, event_type, cb, filters, stateless, extract_as)
 
-    se = self.__get_event_map()
-    se[event_id] = (cb, event_type, attr_name)
+    with self.__get_event_map_lock():
+        se = self.__get_event_map()
+        evt_data = se.get(event_id)
+        if evt_data is not None:
+            desc = "Internal PyTango error:\n" \
+                   "%s.subscribe_event(%s, %s) already has key %d assigned to (%s, %s)\n" \
+                   "Please report error to PyTango" % \
+                   (self, attr_name, event_type, event_id, evt_data[2], evt_data[1])
+            Except.throw_exception("Py_InternalError", desc, "DeviceProxy.subscribe_event")
+        se[event_id] = (cb, event_type, attr_name)
     return event_id
 
 def __DeviceProxy__unsubscribe_event(self, event_id):
@@ -727,20 +766,20 @@ def __DeviceProxy__unsubscribe_event(self, event_id):
 
         Throws     : EventSystemFailed
     """
-    se = self.__get_event_map()
-    if event_id not in se:
-        raise IndexError("This device proxy does not own this subscription " + str(event_id))
+    with self.__get_event_map_lock():
+        se = self.__get_event_map()
+        if event_id not in se:
+            raise IndexError("This device proxy does not own this subscription " + str(event_id))
+        del se[event_id]
     self.__unsubscribe_event(event_id)
-    del se[event_id]
 
 def __DeviceProxy__unsubscribe_event_all(self):
-    se = self.__get_event_map()
-    for event_id in se:
-        try:
-            self.__unsubscribe_event(event_id)
-        except Exception:
-            pass # @todo print or something, but not rethrow
-    se.clear()
+    with self.__get_event_map_lock():
+        se = self.__get_event_map()
+        event_ids = list(se.keys())
+        se.clear()
+    for event_id in event_ids:
+        self.__unsubscribe_event(event_id)
 
 def __DeviceProxy__get_events(self, event_id, callback=None, extract_as=ExtractAs.Numpy):
     """
@@ -793,29 +832,37 @@ def __DeviceProxy__get_events(self, event_id, callback=None, extract_as=ExtractA
         else:
             assert (False)
             raise ValueError("Unknown event_type: " + str(event_type))
-    elif callable(callback):
+    elif isinstance(callback, collections.Callable):
         cb = __CallBackPushEvent()
         cb.push_event = callback
         return self.__get_callback_events(event_id, cb, extract_as)
-    elif hasattr(callback, 'push_event') and callable(callback.push_event):
+    elif hasattr(callback, 'push_event') and isinstance(callback.push_event, collections.Callable):
         cb = __CallBackPushEvent()
         cb.push_event = callback.push_event
         return self.__get_callback_events(event_id, cb, extract_as)
     else:
         raise TypeError("Parameter 'callback' should be None, a callable object or an object with a 'push_event' method.")
 
-def __DeviceProxy__str(self):
-    if not hasattr(self, '_dev_class'):
+def __DeviceProxy___get_info_(self):
+    """Protected method that gets device info once and stores it in cache"""
+    if not hasattr(self, '_dev_info'):
         try:
-            self.__dict__["_dev_class"] = self.info().dev_class
+            self.__dict__["_dev_info"] = self.info()
         except:
-            return "DeviceProxy(%s)" % self.dev_name()
-    return "%s(%s)" % (self._dev_class, self.dev_name())
-    
+            return __TangoInfo()
+    return self._dev_info
+
+def __DeviceProxy__str(self):
+    info = self._get_info_()
+    return "%s(%s)" % (info.dev_class, self.dev_name())
+
+def __DeviceProxy__str(self):
+    info = self._get_info_()
+    return "%s(%s)" % (info.dev_class, self.dev_name())
+
 def __init_DeviceProxy():
     DeviceProxy.__getattr__ = __DeviceProxy__getattr
     DeviceProxy.__setattr__ = __DeviceProxy__setattr
-    DeviceProxy.__del__ = __DeviceProxy__del
     DeviceProxy.__getitem__ = __DeviceProxy__getitem
     DeviceProxy.__setitem__ = __DeviceProxy__setitem
     DeviceProxy.__contains__ = __DeviceProxy__contains
@@ -843,13 +890,17 @@ def __init_DeviceProxy():
     DeviceProxy.set_attribute_config = __DeviceProxy__set_attribute_config
 
     DeviceProxy.__get_event_map = __DeviceProxy__get_event_map
+    DeviceProxy.__get_event_map_lock = __DeviceProxy__get_event_map_lock
     DeviceProxy.subscribe_event = __DeviceProxy__subscribe_event
     DeviceProxy.unsubscribe_event = __DeviceProxy__unsubscribe_event
     DeviceProxy.__unsubscribe_event_all = __DeviceProxy__unsubscribe_event_all
     DeviceProxy.get_events = __DeviceProxy__get_events
     DeviceProxy.__str__ = __DeviceProxy__str
     DeviceProxy.__repr__ = __DeviceProxy__str
+    
+    DeviceProxy._get_info_ = __DeviceProxy___get_info_
 
+    
 def __doc_DeviceProxy():
     def document_method(method_name, desc, append=True):
         return __document_method(DeviceProxy, method_name, desc, append)
@@ -878,12 +929,12 @@ def __doc_DeviceProxy():
         Return     : (DeviceInfo) object
         Example    :
                 dev_info = dev.info()
-                print dev_info.dev_class
-                print dev_info.server_id
-                print dev_info.server_host
-                print dev_info.server_version
-                print dev_info.doc_url
-                print dev_info.dev_type
+                print(dev_info.dev_class)
+                print(dev_info.server_id)
+                print(dev_info.server_host)
+                print(dev_info.server_version)
+                print(dev_info.doc_url)
+                print(dev_info.dev_type)
 
             All DeviceInfo fields are strings except for the server_version
             which is an integer"
@@ -974,7 +1025,7 @@ def __doc_DeviceProxy():
                      command and from which client computer the command
                      was executed
         Example :
-                print black_box(4)
+                print(black_box(4))
     """ )
 
 #-------------------------------------
@@ -992,13 +1043,13 @@ def __doc_DeviceProxy():
         Throws     : ConnectionFailed, CommunicationFailed, DevFailed from device
         Example :
                 com_info = dev.command_query(""DevString"")
-                print com_info.cmd_name
-                print com_info.cmd_tag
-                print com_info.in_type
-                print com_info.out_type
-                print com_info.in_type_desc
-                print com_info.out_type_desc
-                print com_info.disp_level
+                print(com_info.cmd_name)
+                print(com_info.cmd_tag)
+                print(com_info.in_type)
+                print(com_info.out_type)
+                print(com_info.in_type_desc)
+                print(com_info.out_type_desc)
+                print(com_info.disp_level)
                 
         See CommandInfo documentation string form more detail
     """ )
@@ -1021,10 +1072,10 @@ def __doc_DeviceProxy():
         Return     : (DbDevImportInfo)
         Example :
                 dev_import = dev.import_info()
-                print dev_import.name
-                print dev_import.exported
-                print dev_ior.ior
-                print dev_version.version
+                print(dev_import.name)
+                print(dev_import.exported)
+                print(dev_ior.ior)
+                print(dev_version.version)
 
         All DbDevImportInfo fields are strings except for exported which
         is an integer"
@@ -1118,6 +1169,14 @@ def __doc_DeviceProxy():
         value was. Since 7.1.4, it returns a **(format<str>, data<buffer>)**
         unless *extract_as* is String, in which case it returns 
         **(format<str>, data<str>)**.
+
+    .. versionchanged:: 8.0.0
+        For DevEncoded attributes, now returns a DeviceAttribute.value
+        as a tuple **(format<str>, data<bytes>)** unless *extract_as* is String,
+        in which case it returns **(format<str>, data<str>)**. Carefull, if
+        using python >= 3 data<str> is decoded using default python 
+        *utf-8* encoding. This means that PyTango assumes tango DS was written
+        encapsulating string into *utf-8* which is the default python enconding.
     """ )
 
     document_method("read_attributes", """
@@ -1222,6 +1281,7 @@ def __doc_DeviceProxy():
         Parameters : None
         Return     : (sequence<str>) One string for each polled command/attribute.
                      Each string is multi-line string with:
+                     
                         - attribute/command name
                         - attribute/command polling period in milliseconds
                         - attribute/command polling ring buffer
@@ -1412,6 +1472,45 @@ def __doc_DeviceProxy():
 
         New in PyTango 7.0.0
     """ )
+    
+    document_method("write_attribute_reply", """
+    write_attribute_reply(self, id) -> None
+
+            Check if the answer of an asynchronous write_attribute is arrived
+            (polling model). If the reply is arrived and if it is a valid reply,
+            the call returned. If the reply is an exception, it is re-thrown by
+            this call. An exception is also thrown in case of the reply is not
+            yet arrived.
+
+        Parameters :
+            - id : (int) the asynchronous call identifier.
+        Return     : None
+
+        Throws     : AsynCall, AsynReplyNotArrived, CommunicationFailed, DevFailed from device.
+
+        New in PyTango 7.0.0
+
+    write_attribute_reply(self, id, timeout) -> None
+
+            Check if the answer of an asynchronous write_attribute is arrived
+            (polling model). id is the asynchronous call identifier. If the
+            reply is arrived and if it is a valid reply, the call returned. If
+            the reply is an exception, it is re-thrown by this call. If the
+            reply is not yet arrived, the call will wait (blocking the process)
+            for the time specified in timeout. If after timeout milliseconds,
+            the reply is still not there, an exception is thrown. If timeout is
+            set to 0, the call waits until the reply arrived.
+            
+        Parameters :
+            - id      : (int) the asynchronous call identifier.
+            - timeout : (int) the timeout
+            
+        Return     : None
+
+        Throws     : AsynCall, AsynReplyNotArrived, CommunicationFailed, DevFailed from device.
+
+        New in PyTango 7.0.0
+    """ )
 
 #-------------------------------------
 #   Logging administration methods
@@ -1702,7 +1801,7 @@ def __doc_DeviceProxy():
         New in PyTango 7.0.0
     """ )
 
-def init(doc=True):
+def device_proxy_init(doc=True):
     __init_DeviceProxy()
     if doc:
         __doc_DeviceProxy()
