@@ -25,30 +25,27 @@
 This is an internal PyTango module.
 """
 
+from __future__ import print_function
+
 __all__ = [ "ChangeEventProp", "PeriodicEventProp", "ArchiveEventProp",
             "AttributeAlarm", "EventProperties",
-            "AttributeConfig", "AttributeConfig_2", "AttributeConfig_3"]
+            "AttributeConfig", "AttributeConfig_2", "AttributeConfig_3",
+            "MultiAttrProp",
+            "device_server_init"]
 
 __docformat__ = "restructuredtext"
 
 import copy
-import operator
-import types
 
-import _PyTango
-from _PyTango import StdStringVector, StdDoubleVector
-from _PyTango import DeviceImpl, Device_3Impl, Device_4Impl
-from _PyTango import Attribute, WAttribute, MultiAttribute
-from _PyTango import Attr
-from _PyTango import Logger
-from _PyTango import AttrWriteType, AttrDataFormat, DispLevel
-from _PyTango import UserDefaultAttrProp
+from ._PyTango import DevFailed, DeviceImpl, Device_3Impl, Device_4Impl, \
+    Attribute, WAttribute, MultiAttribute, MultiClassAttribute, \
+    Attr, Logger, AttrWriteType, AttrDataFormat, DispLevel, UserDefaultAttrProp
 
-from utils import seq_2_StdStringVector, seq_2_StdDoubleVector
-from utils import document_method as __document_method
-from utils import copy_doc
+from .utils import document_method as __document_method
+from .utils import copy_doc
+from .attr_data import AttrData
 
-import log4tango
+from .log4tango import TangoStream
 
 class AttributeAlarm(object):
     """This class represents the python interface for the Tango IDL object
@@ -99,6 +96,32 @@ class EventProperties(object):
         self.per_event = PeriodicEventProp()
         self.arch_event = ArchiveEventProp()
 
+class MultiAttrProp(object):
+    """This class represents the python interface for the Tango IDL object
+    MultiAttrProp."""
+    
+    def __init__(self):
+        self.label = ''
+        self.description = ''
+        self.unit = ''
+        self.standard_unit = ''
+        self.display_unit = ''
+        self.format = ''
+        self.min_value = ''
+        self.max_value = ''
+        self.min_alarm = ''
+        self.max_alarm = ''
+        self.min_warning = ''
+        self.max_warning = ''
+        self.delta_t = ''
+        self.delta_val = ''
+        self.event_period = ''
+        self.archive_period = ''
+        self.rel_change = ''
+        self.abs_change = ''
+        self.archive_rel_change = ''
+        self.archive_abs_change = ''
+
 def _init_attr_config(attr_cfg):
     """Helper function to initialize attribute config objects"""
     attr_cfg.name = ''
@@ -165,9 +188,12 @@ def __Attribute__get_properties(self, attr_cfg = None):
             New in PyTango 7.1.4
     """
 
-    if attr_cfg is None:
-        attr_cfg = AttributeConfig()
-    return self._get_properties(attr_cfg)
+    if isinstance(attr_cfg,MultiAttrProp):
+        return self._get_properties_multi_attr_prop(attr_cfg)
+    else:
+        if attr_cfg is None:
+            attr_cfg = AttributeConfig()
+        return self._get_properties(attr_cfg)
 
 def __Attribute__get_properties_2(self, attr_cfg = None):
     """get_properties_2(self, attr_cfg = None) -> AttributeConfig_2
@@ -211,7 +237,7 @@ def __Attribute__get_properties_3(self, attr_cfg = None):
         attr_cfg = AttributeConfig_3()
     return self._get_properties_3(attr_cfg)
 
-def __Attribute__set_properties(self, attr_cfg, dev):
+def __Attribute__set_properties(self, attr_cfg, dev = None):
     """set_properties(self, attr_cfg, dev) -> None
 
                 Set attribute properties.
@@ -226,10 +252,14 @@ def __Attribute__set_properties(self, attr_cfg, dev):
 
             New in PyTango 7.1.4
     """
-    if isinstance(attr_cfg, AttributeConfig_3):
-        self._set_properties_3(attr_cfg, dev)
+    
+    if isinstance(attr_cfg,MultiAttrProp):
+        return self._set_properties_multi_attr_prop(attr_cfg)
     else:
-        self._set_properties(attr_cfg, dev)
+        if isinstance(attr_cfg, AttributeConfig_3):
+            self._set_properties_3(attr_cfg, dev)
+        else:
+            self._set_properties(attr_cfg, dev)
 
 
 def __DeviceImpl__get_device_class(self):
@@ -264,16 +294,17 @@ def __DeviceImpl__get_device_properties(self, ds_class = None):
         self.device_property_list = copy.deepcopy(ds_class.device_property_list)
         class_prop = ds_class.class_property_list
         pu.get_device_properties(self, class_prop, self.device_property_list)
-        for prop_name in class_prop.keys():
+        for prop_name in class_prop:
             setattr(self, prop_name, pu.get_property_values(prop_name, class_prop))
-        for prop_name in self.device_property_list.keys():
+        for prop_name in self.device_property_list:
             setattr(self, prop_name, self.prop_util.get_property_values(prop_name, self.device_property_list))
-    except _PyTango.DevFailed, e:
-        print "----> ", e
-        raise e
+    except DevFailed as df:
+        print(80*"-")
+        print(df)
+        raise df
 
 def __DeviceImpl__add_attribute(self, attr, r_meth=None, w_meth=None, is_allo_meth=None):
-    """add_attribute(self, attr, r_meth=None, w_meth=None, is_allo_meth=None) -> None
+    """add_attribute(self, attr, r_meth=None, w_meth=None, is_allo_meth=None) -> Attr
 
             Add a new attribute to the device attribute list. Please, note that if you add
             an attribute to a device at device creation time, this attribute will be added
@@ -281,43 +312,57 @@ def __DeviceImpl__add_attribute(self, attr, r_meth=None, w_meth=None, is_allo_me
             same class created after this attribute addition will also have this attribute.
 
         Parameters :
-            attr : (Attr) the new attribute to be added to the list.
-            r_meth : (callable) the read method to be called on a read request
-            w_meth : (callable) the write method to be called on a write request (if attr is writable)
-            is_allo_meth: (callable) the method that is called to check if it is possible to access the attribute or not
+            - attr : (Attr or AttrData) the new attribute to be added to the list.
+            - r_meth : (callable) the read method to be called on a read request
+            - w_meth : (callable) the write method to be called on a write request
+                       (if attr is writable)
+            - is_allo_meth: (callable) the method that is called to check if it
+                            is possible to access the attribute or not
         
-        Return     : None
+        Return     : (Attr) the newly created attribute.
         
         Throws     : DevFailed"""
+    
+    attr_data = None
+    if isinstance(attr, AttrData):
+        attr_data = attr
+        attr = attr.to_attr()
+    
     att_name = attr.get_name()
 
     add_name_in_list = False
-    if r_meth is not None:
-        r_meth_name = 'read_%s' % att_name
-        if not hasattr(self.__class__, r_meth_name):
-            setattr(self.__class__, r_meth_name, r_meth)
-            add_name_in_list = True
-
-    if w_meth is not None:
-        w_meth_name = 'write_%s' % att_name
-        if not hasattr(self.__class__, w_meth_name):
-            setattr(self.__class__, w_meth_name, w_meth)
-            add_name_in_list = True
-
-    if is_allo_meth is not None:
-        allo_meth_name = 'is_%s_allowed' % att_name
-        if not hasattr(self.__class__, allo_meth_name):
-            setattr(self.__class__, allo_meth_name,is_allo_meth)
-            add_name_in_list = True
-
+    
+    r_name = 'read_%s' % att_name
+    if r_meth is None:
+        if attr_data is not None:
+            r_name = attr_data.read_method_name
+    else:
+        r_name = r_meth.__name__
+        
+    w_name = 'write_%s' % att_name
+    if w_meth is None:
+        if attr_data is not None:
+            w_name = attr_data.write_method_name
+    else:
+        w_name = w_meth.__name__
+    
+    ia_name = 'is_%s_allowed' % att_name
+    if is_allo_meth is None:
+        if attr_data is not None:
+            ia_name = attr_data.is_allowed_name
+    else:
+        ia_name = is_allo_meth.__name__
+    
     try:
-        self._add_attribute(attr)
+        self._add_attribute(attr, r_name, w_name, ia_name)
         if add_name_in_list:
             cl = self.get_device_class()
             cl.dyn_att_added_methods.append(att_name)
     except:
         if add_name_in_list:
             self._remove_attr_meth(att_name)
+        raise
+    return attr
 
 def __DeviceImpl__remove_attribute(self, attr_name):
     """
@@ -361,14 +406,14 @@ def __DeviceImpl___remove_attr_meth(self,attr_name):
         if hasattr(self.__class__, r_meth_name):
             delattr(self.__class__, r_meth_name)
 
-    w_meth_name = 'write_%s' % attr_name
-    if hasattr(self.__class__, w_meth_name):
-        delattr(self.__class__, w_meth_name)
+        w_meth_name = 'write_%s' % attr_name
+        if hasattr(self.__class__, w_meth_name):
+            delattr(self.__class__, w_meth_name)
 
-    allo_meth_name = 'is_%s_allowed' % attr_name
-    if hasattr(self.__class__, allo_meth_name):
-        delattr(self.__class__, allo_meth_name)
-    cl.dyn_att_added_methods.remove(attr_name)
+        allo_meth_name = 'is_%s_allowed' % attr_name
+        if hasattr(self.__class__, allo_meth_name):
+            delattr(self.__class__, allo_meth_name)
+        cl.dyn_att_added_methods.remove(attr_name)
 
 def __join_msg(msg):
     return ' '.join(map(str, msg))
@@ -381,7 +426,7 @@ def __DeviceImpl__debug_stream(self, *msg):
 
             Since PyTango 7.1.3, the same can be achieved with::
             
-                print >>self.log_debug, msg
+                print(msg, file=self.log_debug)
             
         Parameters :
             - msg : (str) the message to be sent to the debug stream
@@ -397,7 +442,7 @@ def __DeviceImpl__info_stream(self, *msg):
 
             Since PyTango 7.1.3, the same can be achieved with::
             
-                print >>self.log_info, msg
+                print(msg, file=self.log_info)
 
         Parameters :
             - msg : (str) the message to be sent to the info stream
@@ -413,7 +458,7 @@ def __DeviceImpl__warn_stream(self, *msg):
 
             Since PyTango 7.1.3, the same can be achieved with::
             
-                print >>self.log_warn, msg
+                print(msg, file=self.log_warn)
 
         Parameters :
             - msg : (str) the message to be sent to the warn stream
@@ -429,7 +474,7 @@ def __DeviceImpl__error_stream(self, *msg):
 
             Since PyTango 7.1.3, the same can be achieved with::
             
-                print >>self.log_error, msg
+                print(msg, file=self.log_error)
 
         Parameters :
             - msg : (str) the message to be sent to the error stream
@@ -445,7 +490,7 @@ def __DeviceImpl__fatal_stream(self, *msg):
 
             Since PyTango 7.1.3, the same can be achieved with::
             
-                print >>self.log_fatal, msg
+                print(msg, file=self.log_fatal)
 
         Parameters :
             - msg : (str) the message to be sent to the fatal stream
@@ -456,31 +501,31 @@ def __DeviceImpl__fatal_stream(self, *msg):
 @property
 def __DeviceImpl__debug(self):
     if not hasattr(self, "_debug_s"):
-        self._debug_s = log4tango.TangoStream(self.debug_stream)
+        self._debug_s = TangoStream(self.debug_stream)
     return self._debug_s
 
 @property
 def __DeviceImpl__info(self):
     if not hasattr(self, "_info_s"):
-        self._info_s = log4tango.TangoStream(self.info_stream)
+        self._info_s = TangoStream(self.info_stream)
     return self._info_s
 
 @property
 def __DeviceImpl__warn(self):
     if not hasattr(self, "_warn_s"):
-        self._warn_s = log4tango.TangoStream(self.warn_stream)
+        self._warn_s = TangoStream(self.warn_stream)
     return self._warn_s
 
 @property
 def __DeviceImpl__error(self):
     if not hasattr(self, "_error_s"):
-        self._error_s = log4tango.TangoStream(self.error_stream)
+        self._error_s = TangoStream(self.error_stream)
     return self._error_s
 
 @property
 def __DeviceImpl__fatal(self):
     if not hasattr(self, "_fatal_s"):
-        self._fatal_s = log4tango.TangoStream(self.fatal_stream)
+        self._fatal_s = TangoStream(self.fatal_stream)
     return self._fatal_s
 
 def __DeviceImpl__str(self):
@@ -610,7 +655,7 @@ def __init_Attribute():
     Attribute.get_properties_2 = __Attribute__get_properties_2
     Attribute.get_properties_3 = __Attribute__get_properties_3
     Attribute.set_properties = __Attribute__set_properties
-    
+
 def __init_Logger():
     Logger.log = __Logger__log
     Logger.log_unconditionally = __Logger__log_unconditionally
@@ -1090,7 +1135,52 @@ def __doc_DeviceImpl():
         
         New in PyTango 7.1.2
     """ )
+    
+    document_method("get_min_poll_period", """
+    get_min_poll_period(self) -> int
 
+            Returns the min poll period
+
+        Parameters : None
+        Return     : (int) the min poll period
+        
+        New in PyTango 7.2.0
+    """ )
+
+    document_method("get_cmd_min_poll_period", """
+    get_cmd_min_poll_period(self) -> seq<str>
+
+            Returns the min command poll period
+
+        Parameters : None
+        Return     : (seq<str>) the min command poll period
+        
+        New in PyTango 7.2.0
+    """ )
+
+    document_method("get_attr_min_poll_period", """
+    get_attr_min_poll_period(self) -> seq<str>
+
+            Returns the min attribute poll period
+
+        Parameters : None
+        Return     : (seq<str>) the min attribute poll period
+        
+        New in PyTango 7.2.0
+    """ )
+
+    document_method("push_att_conf_event", """
+    push_att_conf_event(self, attr) -> None
+
+            Push an attribute configuration event.
+
+        Parameters : (Attribute) the attribute for which the configuration event
+                     will be sent.
+        Return     : None
+        
+        New in PyTango 7.2.1
+    """ )
+    
 def __doc_extra_DeviceImpl(cls):
     def document_method(method_name, desc, append=True):
         return __document_method(cls, method_name, desc, append)
@@ -1155,7 +1245,7 @@ def __doc_extra_DeviceImpl(cls):
 
         Throws     : DevFailed This method does not throw exception but a redefined method can.
     """ )
-
+    
     copy_doc(cls, "dev_state")
     copy_doc(cls, "dev_status")
 
@@ -1569,6 +1659,30 @@ def __doc_Attribute():
         New in PyTango 7.1.0
     """ )
 
+    document_method("set_data_ready_event", """
+    set_data_ready_event(self, implemented) -> None
+
+            Set a flag to indicate that the server fires data ready events.
+
+        Parameters :
+            - implemented : (bool) True when the server fires data ready events manually. 
+        Return     : None
+        
+        New in PyTango 7.2.0
+    """ )
+
+    document_method("is_data_ready_event", """
+    is_data_ready_event(self) -> bool
+
+            Check if the data ready event is fired manually (without polling)
+            for this attribute. 
+
+        Parameters : None
+        Return     : (bool) True if a manual fire data ready event is implemented. 
+        
+        New in PyTango 7.2.0
+    """ )
+    
     document_method("remove_configuration", """
     remove_configuration(self) -> None
 
@@ -1586,6 +1700,7 @@ def __doc_Attribute():
         New in PyTango 7.1.0
     """ )
 
+
 def __doc_WAttribute():
     def document_method(method_name, desc, append=True):
         return __document_method(WAttribute, method_name, desc, append)
@@ -1690,21 +1805,73 @@ def __doc_WAttribute():
         Return     : (obj) the attribute write value.
     """ )
 
+def __doc_MultiClassAttribute():
+    def document_method(method_name, desc, append=True):
+        return __document_method(MultiClassAttribute, method_name, desc, append)
+
+    MultiClassAttribute.__doc__ = """
+    There is one instance of this class for each device class.
+    This class is mainly an aggregate of :class:`~PyTango.Attr` objects. 
+    It eases management of multiple attributes
+    
+    New in PyTango 7.2.1"""
+    
+    document_method("get_attr", """
+    get_attr(self, attr_name) -> Attr
+
+            Get the :class:`~PyTango.Attr` object for the attribute with
+            name passed as parameter
+
+        Parameters :
+            - attr_name : (str) attribute name
+        Return     : (Attr) the attribute object
+        
+        Throws     : DevFailed If the attribute is not defined.
+        
+        New in PyTango 7.2.1
+    """ )
+
+    document_method("remove_attr", """
+    remove_attr(self, attr_name, cl_name) -> None
+
+            Remove the :class:`~PyTango.Attr` object for the attribute with
+            name passed as parameter. Does nothing if the attribute does not
+            exist.
+
+        Parameters :
+            - attr_name : (str) attribute name
+            - cl_name : (str) the attribute class name
+        
+        New in PyTango 7.2.1
+    """ )
+
+    document_method("get_attr_list", """
+    get_attr_list(self) -> seq<Attr>
+
+            Get the list of :class:`~PyTango.Attr` for this device class.
+
+        Return     : (seq<Attr>) the list of attribute objects
+        
+        New in PyTango 7.2.1
+    """ )
+
 def __doc_MultiAttribute():
     def document_method(method_name, desc, append=True):
         return __document_method(MultiAttribute, method_name, desc, append)
 
     MultiAttribute.__doc__ = """
     There is one instance of this class for each device.
-    This class is mainly an aggregate of Attribute or WAttribute objects. 
-    It eases management of multiple attributes"""
+    This class is mainly an aggregate of :class:`~PyTango.Attribute` or
+    :class:`~PyTango.WAttribute` objects. It eases management of multiple
+    attributes"""
     
     document_method("get_attr_by_name", """
     get_attr_by_name(self, attr_name) -> Attribute
 
-            Get Attribute object from its name.
-            This method returns an Attribute object with a name passed as 
-            parameter. The equality on attribute name is case independant.
+            Get :class:`~PyTango.Attribute` object from its name.
+            This method returns an :class:`~PyTango.Attribute` object with a
+            name passed as parameter. The equality on attribute name is case
+            independant.
 
         Parameters :
             - attr_name : (str) attribute name
@@ -1716,9 +1883,9 @@ def __doc_MultiAttribute():
     document_method("get_attr_by_ind", """
     get_attr_by_ind(self, ind) -> Attribute
 
-            Get Attribute object from its index.
-            This method returns an Attribute object from the index in the 
-            main attribute vector.
+            Get :class:`~PyTango.Attribute` object from its index.
+            This method returns an :class:`~PyTango.Attribute` object from the
+            index in the main attribute vector.
 
         Parameters :
             - ind : (int) the attribute index
@@ -1729,8 +1896,9 @@ def __doc_MultiAttribute():
     get_w_attr_by_name(self, attr_name) -> WAttribute
 
             Get a writable attribute object from its name.
-            This method returns an WAttribute object with a name passed as 
-            parameter. The equality on attribute name is case independant.
+            This method returns an :class:`~PyTango.WAttribute` object with a
+            name passed as parameter. The equality on attribute name is case
+            independant.
 
         Parameters :
             - attr_name : (str) attribute name
@@ -1743,8 +1911,8 @@ def __doc_MultiAttribute():
     get_w_attr_by_ind(self, ind) -> WAttribute
 
             Get a writable attribute object from its index.
-            This method returns an WAttribute object from the index in the 
-            main attribute vector.
+            This method returns an :class:`~PyTango.WAttribute` object from the
+            index in the main attribute vector.
 
         Parameters :
             - ind : (int) the attribute index
@@ -1756,8 +1924,8 @@ def __doc_MultiAttribute():
 
             Get Attribute index into the main attribute vector from its name.
             This method returns the index in the Attribute vector (stored in the 
-            MultiAttribute object) of an attribute with a given name. 
-            The name equality is case independant.
+            :class:`~PyTango.MultiAttribute` object) of an attribute with a
+            given name. The name equality is case independant.
 
         Parameters :
             - attr_name : (str) attribute name
@@ -1812,6 +1980,16 @@ def __doc_MultiAttribute():
         New in PyTango 7.0.0
     """ )
     
+    document_method("get_attribute_list", """
+    get_attribute_list(self) -> seq<Attribute>
+
+            Get the list of attribute objects.
+
+        Return     : (seq<Attribute>) list of attribute objects
+    
+        New in PyTango 7.2.1
+    """ )
+    
 def __doc_Attr():
     def document_method(method_name, desc, append=True):
         return __document_method(Attr, method_name, desc, append)
@@ -1946,6 +2124,29 @@ def __doc_Attr():
         Return     : (bool) true if a archive event criteria will be checked.
     """ )
 
+    document_method("set_data_ready_event", """
+    set_data_ready_event(self, implemented) -> None
+
+            Set a flag to indicate that the server fires data ready events.
+
+        Parameters :
+            - implemented : (bool) True when the server fires data ready events
+        Return     : None
+        
+        New in PyTango 7.2.0
+    """ )
+
+    document_method("is_data_ready_event", """
+    is_data_ready_event(self) -> bool
+
+            Check if the data ready event is fired for this attribute.
+
+        Parameters : None
+        Return     : (bool) true if firing data ready event is implemented.
+        
+        New in PyTango 7.2.0
+    """ )
+
     document_method("get_name", """
     get_name(self) -> str
 
@@ -2037,6 +2238,29 @@ def __doc_Attr():
         Return     : (bool) if it is assoc
     """ )
 
+    document_method("get_cl_name", """
+    get_cl_name(self) -> str
+
+            Returns the class name
+
+        Parameters : None
+        Return     : (str) the class name
+        
+        New in PyTango 7.2.0
+    """ )
+
+    document_method("set_cl_name", """
+    set_cl_name(self, cl) -> None
+
+            Sets the class name
+
+        Parameters :
+            - cl : (str) new class name
+        Return     : None
+        
+        New in PyTango 7.2.0
+    """ )
+    
     document_method("get_class_properties", """
     get_class_properties(self) -> sequence<AttrProperty>
 
@@ -2219,66 +2443,150 @@ def __doc_UserDefaultAttrProp():
     """ )
     
     document_method("set_abs_change", """
-    set_abs_change(self, def_abs_change) -> None
+    set_abs_change(self, def_abs_change) -> None <= DEPRECATED
 
             Set default change event abs_change property. 
 
         Parameters :  
             - def_abs_change : (str) the user default change event abs_change property 
         Return     : None
+        
+        Deprecated since PyTango 8.0. Please use set_event_abs_change instead.
+    """ )
+
+    document_method("set_event_abs_change", """
+    set_event_abs_change(self, def_abs_change) -> None
+
+            Set default change event abs_change property. 
+
+        Parameters :  
+            - def_abs_change : (str) the user default change event abs_change property 
+        Return     : None
+        
+        New in PyTango 8.0
     """ )
     
     document_method("set_rel_change", """
-    set_rel_change(self, def_rel_change) -> None
+    set_rel_change(self, def_rel_change) -> None <= DEPRECATED
 
             Set default change event rel_change property. 
 
         Parameters :  
             - def_rel_change : (str) the user default change event rel_change property 
         Return     : None
+        
+        Deprecated since PyTango 8.0. Please use set_event_rel_change instead.
+    """ )
+
+    document_method("set_event_rel_change", """
+    set_event_rel_change(self, def_rel_change) -> None
+
+            Set default change event rel_change property. 
+
+        Parameters :  
+            - def_rel_change : (str) the user default change event rel_change property 
+        Return     : None
+        
+        New in PyTango 8.0
     """ )
     
     document_method("set_period", """ 
-    set_period(self, def_period) -> None
+    set_period(self, def_period) -> None <= DEPRECATED
 
             Set default periodic event period property. 
 
         Parameters :  
             - def_period : (str) the user default periodic event period property 
         Return     : None
+        
+        Deprecated since PyTango 8.0. Please use set_event_period instead.
     """ )
 
+    document_method("set_event_period", """ 
+    set_event_period(self, def_period) -> None
+
+            Set default periodic event period property. 
+
+        Parameters :  
+            - def_period : (str) the user default periodic event period property 
+        Return     : None
+        
+        New in PyTango 8.0
+    """ )
+    
     document_method("set_archive_abs_change", """
-    set_archive_abs_change(self, def_archive_abs_change) -> None
+    set_archive_abs_change(self, def_archive_abs_change) -> None <= DEPRECATED
 
             Set default archive event abs_change property. 
 
         Parameters :  
             - def_archive_abs_change : (str) the user default archive event abs_change property 
         Return     : None
+        
+        Deprecated since PyTango 8.0. Please use set_archive_event_abs_change instead.
     """ )
 
+    document_method("set_archive_event_abs_change", """
+    set_archive_event_abs_change(self, def_archive_abs_change) -> None
+
+            Set default archive event abs_change property. 
+
+        Parameters :  
+            - def_archive_abs_change : (str) the user default archive event abs_change property 
+        Return     : None
+        
+        New in PyTango 8.0
+    """ )
+    
     document_method("set_archive_rel_change", """
-    set_archive_rel_change(self, def_archive_rel_change) -> None
+    set_archive_rel_change(self, def_archive_rel_change) -> None <= DEPRECATED
 
             Set default archive event rel_change property. 
 
         Parameters :  
             - def_archive_rel_change : (str) the user default archive event rel_change property 
         Return     : None
+        
+        Deprecated since PyTango 8.0. Please use set_archive_event_rel_change instead.
+    """ )
+
+    document_method("set_archive_event_rel_change", """
+    set_archive_event_rel_change(self, def_archive_rel_change) -> None
+
+            Set default archive event rel_change property. 
+
+        Parameters :  
+            - def_archive_rel_change : (str) the user default archive event rel_change property 
+        Return     : None
+        
+        New in PyTango 8.0
     """ )
     
     document_method("set_archive_period", """
-    set_archive_period(self, def_archive_period) -> None
+    set_archive_period(self, def_archive_period) -> None <= DEPRECATED
 
             Set default archive event period property. 
 
         Parameters :  
             - def_archive_period : (str) t
         Return     : None
+        
+        Deprecated since PyTango 8.0. Please use set_archive_event_period instead.
+    """ )
+
+    document_method("set_archive_event_period", """
+    set_archive_event_period(self, def_archive_period) -> None
+
+            Set default archive event period property. 
+
+        Parameters :  
+            - def_archive_period : (str) t
+        Return     : None
+        
+        New in PyTango 8.0
     """ )
     
-def init(doc=True):
+def device_server_init(doc=True):
     __init_DeviceImpl()
     __init_Attribute()
     __init_Attr()
@@ -2290,5 +2598,6 @@ def init(doc=True):
         __doc_Attribute()
         __doc_WAttribute()
         __doc_MultiAttribute()
+        __doc_MultiClassAttribute()
         __doc_UserDefaultAttrProp()
         __doc_Attr()
