@@ -1,48 +1,48 @@
-#############################################################################
+################################################################################
 ##
 ## This file is part of PyTango, a python binding for Tango
-##
+## 
 ## http://www.tango-controls.org/static/PyTango/latest/doc/html/index.html
 ##
-## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
-##
-## This is free software; you can redistribute it and/or modify
+## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
+## 
+## PyTango is free software: you can redistribute it and/or modify
 ## it under the terms of the GNU Lesser General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or
+## the Free Software Foundation, either version 3 of the License, or
 ## (at your option) any later version.
-##
-## This software is distributed in the hope that it will be useful,
+## 
+## PyTango is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
 ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ## GNU Lesser General Public License for more details.
-##
+## 
 ## You should have received a copy of the GNU Lesser General Public License
-## along with this program; if not, see <http://www.gnu.org/licenses/>.
-###########################################################################
+## along with PyTango.  If not, see <http://www.gnu.org/licenses/>.
+##
+################################################################################
 
 """
 This is an internal PyTango module.
 """
 
-__all__ = []
-            
+__all__ = ["group_element_init"]
+
 __docformat__ = "restructuredtext"
 
 import operator
-import types
 
-from _PyTango import StdStringVector
-from _PyTango import GroupElement
+from ._PyTango import StdStringVector, GroupElement
 
-from utils import document_method as __document_method
-from utils import seq_2_StdStringVector
+from .utils import document_method as __document_method
+from .utils import seq_2_StdStringVector
+import collections
 
 def __apply_to(fn, key):
     if isinstance(key, slice):
         if key.step:
-            return [ fn(x) for x in xrange(key.start, key.stop, key.step) ]
+            return [ fn(x) for x in range(key.start, key.stop, key.step) ]
         else:
-            return [ fn(x) for x in xrange(key.start, key.stop) ]
+            return [ fn(x) for x in range(key.start, key.stop) ]
     else:
         return fn(key)
 
@@ -73,7 +73,7 @@ def __GroupElement__add(self, patterns_or_group, timeout_ms=-1):
         return self.__add(patterns_or_group, timeout_ms)
     elif isinstance(patterns_or_group, str):
         return self.__add(patterns_or_group, timeout_ms)
-    elif operator.isSequenceType(patterns_or_group):
+    elif isinstance(patterns_or_group, collections.Sequence):
         patterns = seq_2_StdStringVector(patterns_or_group)
         return self.__add(patterns, timeout_ms)
     else:
@@ -82,7 +82,7 @@ def __GroupElement__add(self, patterns_or_group, timeout_ms=-1):
 def __GroupElement__remove(self, patterns, forward=True):
     if isinstance(patterns, str):
         return self.__remove(patterns, forward)
-    elif operator.isSequenceType(patterns):
+    elif isinstance(patterns, collections.Sequence):
         std_patterns = seq_2_StdStringVector(patterns)
         return self.__remove(std_patterns, forward)
     else:
@@ -189,13 +189,14 @@ def __doc_GroupElement():
         
         Parameters :
             - patterns   : (str | sequence<str>) A string with the pattern or a
-                            list of patterns.
+                           list of patterns.
             - forward    : (bool) If fwd is set to true (the default), the remove
-                            request is also forwarded to subgroups. Otherwise,
-                            it is only applied to the local set of elements.
-                            For instance, the following code remove any
-                            stepper motor in the hierarchy:
-                                root_group->remove("*/stepper_motor/*");
+                           request is also forwarded to subgroups. Otherwise,
+                           it is only applied to the local set of elements.
+                           For instance, the following code remove any
+                           stepper motor in the hierarchy:
+                           
+                               root_group->remove("*/stepper_motor/*");
                 
         Return     : None
         
@@ -578,7 +579,7 @@ def __doc_GroupElement():
     # get_parent(self)
     
 
-def init(doc=True):
+def group_element_init(doc=True):
     __init_GroupElement()
     if doc:
         __doc_GroupElement()
