@@ -1,30 +1,31 @@
-#############################################################################
+################################################################################
 ##
 ## This file is part of PyTango, a python binding for Tango
-##
+## 
 ## http://www.tango-controls.org/static/PyTango/latest/doc/html/index.html
 ##
-## (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
-##
-## This is free software; you can redistribute it and/or modify
+## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
+## 
+## PyTango is free software: you can redistribute it and/or modify
 ## it under the terms of the GNU Lesser General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or
+## the Free Software Foundation, either version 3 of the License, or
 ## (at your option) any later version.
-##
-## This software is distributed in the hope that it will be useful,
+## 
+## PyTango is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
 ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ## GNU Lesser General Public License for more details.
-##
+## 
 ## You should have received a copy of the GNU Lesser General Public License
-## along with this program; if not, see <http://www.gnu.org/licenses/>.
-###########################################################################
+## along with PyTango.  If not, see <http://www.gnu.org/licenses/>.
+##
+################################################################################
 
 """
 This is an internal PyTango module.
 """
 
-__all__ = []
+__all__ = ["time_val_init"]
 
 __docformat__ = "restructuredtext"
 
@@ -32,7 +33,8 @@ import time
 import datetime
 import operator
 
-from _PyTango import TimeVal
+from ._PyTango import TimeVal
+import numbers
 
 def __TimeVal__init(self, a=None, b=None, c=None):
     TimeVal.__init_original(self)
@@ -43,7 +45,7 @@ def __TimeVal__init(self, a=None, b=None, c=None):
         assert(b is None and c is None)
         a = time.mktime(a.timetuple()) + a.microsecond*1E-6
 
-    elif operator.isNumberType(a):
+    elif isinstance(a, numbers.Number):
         if b is None:
             self.tv_sec = int(a)
             usec = (a - self.tv_sec) * 1E6
@@ -191,6 +193,6 @@ def __init_TimeVal():
     TimeVal.isoformat = __TimeVal__isoformat
     TimeVal.__str__ = __TimeVal__str__
     
-def init(doc=True):
+def time_val_init(doc=True):
     __init_TimeVal()
 
