@@ -21,15 +21,8 @@
    
 *******************************************************************************/
 
-#include <boost/python.hpp>
-#include <boost/python/handle.hpp>
-#include <tango.h>
-#include <iostream>
-#include <string>
-#include <memory>
-
+#include "precompiled_header.hpp"
 #include "device_attribute.h"
-
 #include "pytgutils.h"
 #include "tango_numpy.h"
 #include "fast_from_py.h"
@@ -51,33 +44,35 @@ static const char* has_failed_attr_name = "has_failed";
 
 
 template<long tangoTypeConst>
-struct python_tangocpp {
+struct python_tangocpp
+{
     typedef typename TANGO_const2type(tangoTypeConst) TangoScalarType;
 
-    static inline void to_cpp(const object & py_value, TangoScalarType & result)
+    static inline void to_cpp(const bopy::object & py_value, TangoScalarType & result)
     {
-        result = extract<TangoScalarType>(py_value);
+        result = bopy::extract<TangoScalarType>(py_value);
     }
 
-    static inline object to_python(const TangoScalarType & value)
+    static inline bopy::object to_python(const TangoScalarType & value)
     {
-        return object(value);
+        return bopy::object(value);
     }
 };
 
 template<>
-struct python_tangocpp<Tango::DEV_STRING> {
+struct python_tangocpp<Tango::DEV_STRING>
+{
     static const long tangoTypeConst = Tango::DEV_STRING;
     typedef TANGO_const2type(tangoTypeConst) TangoScalarType;
 
-    static inline void to_cpp(const object & py_value, TangoScalarType & result)
+    static inline void to_cpp(const bopy::object & py_value, TangoScalarType & result)
     {
-        result = CORBA::string_dup(extract<TangoScalarType>(py_value));
+        result = CORBA::string_dup(bopy::extract<TangoScalarType>(py_value));
     }
 
-    static inline object to_python(const TangoScalarType & value)
+    static inline bopy::object to_python(const TangoScalarType & value)
     {
-        return object(std::string(value));
+        return bopy::object(std::string(value));
     }
 };
 
@@ -85,177 +80,278 @@ struct python_tangocpp<Tango::DEV_STRING> {
 #   include "device_attribute_numpy.hpp"
 #endif
 
-namespace PyDeviceAttribute {
+#define EXTRACT_VALUE(self, value_ptr) \
+try { \
+    self >> value_ptr; \
+} catch (Tango::DevFailed &e ) { \
+    if (strcmp(e.errors[0].reason.in(),"API_EmptyDeviceAttribute") != 0) \
+        throw; \
+}
+    
 
-    template<long tangoTypeConst>
-    static inline void _update_value_as_string(Tango::DeviceAttribute &self, object py_value)
+namespace PyDeviceAttribute
+{
+    template<long tangoTypeConst> static inline void
+    _update_value_as_bin(Tango::DeviceAttribute &self,
+                         bopy::object py_value, bool read_only)
     {
         typedef typename TANGO_const2type(tangoTypeConst) TangoScalarType;
         typedef typename TANGO_const2arraytype(tangoTypeConst) TangoArrayType;
 
         // Extract the actual data from Tango::DeviceAttribute (self)
         TangoArrayType* value_ptr = 0;
-        try {
-            self >> value_ptr;
-        } catch (Tango::DevFailed &e ) {
-            if (strcmp(e.errors[0].reason.in(),"API_EmptyDeviceAttribute") != 0)
-                throw;
-        }
-        std::auto_ptr<TangoArrayType> guard_value_ptr(value_ptr);
+        EXTRACT_VALUE(self, value_ptr)
+        unique_pointer<TangoArrayType> guard_value_ptr(value_ptr);
 
-        if (value_ptr == 0) {
-            py_value.attr(value_attr_name) = boost::python::str();
-            py_value.attr(w_value_attr_name) = object();
+        py_value.attr(w_value_attr_name) = bopy::object();
+
+        if (value_ptr == 0)
+        {
+            if (read_only)
+            {
+                py_value.attr(value_attr_name) =
+                    bopy::object(bopy::handle<>(_PyObject_New(&PyBytes_Type)));
+            }
+            else
+            {
+                py_value.attr(value_attr_name) =
+                    bopy::object(bopy::handle<>(_PyObject_New(&PyByteArray_Type)));
+            }
             return;
         }
 
         TangoScalarType* buffer = value_ptr->get_buffer();
 
         const char *ch_ptr = reinterpret_cast<char *>(buffer);
-        size_t nb_bytes = value_ptr->length() * sizeof(TangoScalarType);
+        Py_ssize_t nb_bytes = (Py_ssize_t)value_ptr->length() * sizeof(TangoScalarType);
 
-        py_value.attr(value_attr_name) = str(ch_ptr, (size_t)nb_bytes);
-        py_value.attr(w_value_attr_name) = object();
+        PyObject* data_ptr = NULL;
+        if (read_only)
+        {
+            data_ptr = PyBytes_FromStringAndSize(ch_ptr, nb_bytes);
+        }
+        else
+        {
+            data_ptr = PyByteArray_FromStringAndSize(ch_ptr, nb_bytes);
+        }
+        py_value.attr(value_attr_name) = bopy::object(bopy::handle<>(data_ptr));
     }
 
-    template<>
-    inline void _update_value_as_string<Tango::DEV_ENCODED>(Tango::DeviceAttribute &self, object py_value)
+    template<> inline void
+    _update_value_as_bin<Tango::DEV_ENCODED>(Tango::DeviceAttribute &self,
+                                             bopy::object py_value,
+                                             bool read_only)
     {
-        Tango::DevVarEncodedArray* value;
-        self >> value;
-        std::auto_ptr<Tango::DevVarEncodedArray> guard(value);
+        Tango::DevVarEncodedArray* value_ptr;
+        EXTRACT_VALUE(self, value_ptr)
+        unique_pointer<Tango::DevVarEncodedArray> guard(value_ptr);
         
-        Tango::DevEncoded* buffer = value->get_buffer();
+        Tango::DevEncoded* buffer = value_ptr->get_buffer();
+        Tango::DevEncoded& r_buffer = buffer[0];
+        bopy::str r_encoded_format(r_buffer.encoded_format);
         
-        using namespace boost::python;
-        str encoded_format(buffer[0].encoded_format);
-        str encoded_data((const char*)buffer[0].encoded_data.get_buffer(),
-                          buffer[0].encoded_data.length());
-        
-        py_value.attr(value_attr_name) = make_tuple(encoded_format, encoded_data);
+        Tango::DevVarCharArray& r_encoded_data_array = r_buffer.encoded_data;
+        char* r_ch_ptr = (char*) r_encoded_data_array.get_buffer();
+        Py_ssize_t r_size = r_encoded_data_array.length();
+        PyObject* r_encoded_data_ptr = NULL;
+        if (read_only)
+        {
+            r_encoded_data_ptr = PyBytes_FromStringAndSize(r_ch_ptr, r_size);
+        }
+        else
+        {
+            r_encoded_data_ptr = PyByteArray_FromStringAndSize(r_ch_ptr, r_size);
+        }
+        bopy::object r_encoded_data = bopy::object(bopy::handle<>(r_encoded_data_ptr));
 
-        if (self.get_written_dim_x() > 0) {
-            bool is_write_type = self.get_written_dim_x() && (value->length() < 2);
-            if (is_write_type) {
-                object w_encoded_format(encoded_format);
-                object w_encoded_data(encoded_data);
-                py_value.attr(w_value_attr_name) = make_tuple(w_encoded_format, w_encoded_data);
-            } else {
-                str w_encoded_format(buffer[1].encoded_format);
-                str w_encoded_data((const char*)buffer[1].encoded_data.get_buffer(),
-                                   buffer[1].encoded_data.length());
-                py_value.attr(w_value_attr_name) = make_tuple(w_encoded_format, w_encoded_data);
+        py_value.attr(value_attr_name) =
+            bopy::make_tuple(r_encoded_format, r_encoded_data);
+
+        if (self.get_written_dim_x() > 0)
+        {
+            bool is_write_type = self.get_written_dim_x() && (value_ptr->length() < 2);
+            if (is_write_type)
+            {
+                bopy::object w_encoded_format(r_encoded_format);
+                bopy::object w_encoded_data(r_encoded_data);
+                py_value.attr(w_value_attr_name) =
+                    bopy::make_tuple(w_encoded_format, w_encoded_data);
             }
-        } else {
-            py_value.attr(w_value_attr_name) = object();
-        }
-    }
-    
-    template<long tangoTypeConst>
-    static inline void _update_scalar_values(Tango::DeviceAttribute &self, object py_value)
-    {
-        typedef typename TANGO_const2type(tangoTypeConst) TangoScalarType;
-        
-        if (self.get_written_dim_x() > 0) {
-            std::vector<TangoScalarType> val;
-            self.extract_read(val);
-            // In the following lines, the cast is absolutely necessary because
-            // vector<TangoScalarType> may not be a vector<TangoScalarType> at
-            // compile time. For example, for vector<DevBoolean>, the compiler
-            // may create a std::_Bit_reference type.
-            py_value.attr(value_attr_name) = object((TangoScalarType)val[0]);
-            self.extract_set(val);
-            py_value.attr(w_value_attr_name) = object((TangoScalarType)val[0]);
-        } else {
-            TangoScalarType rvalue;
-            self >> rvalue;
-            py_value.attr(value_attr_name) = object(rvalue);
-            py_value.attr(w_value_attr_name) = object();
-        }
-    }
+            else 
+            {
+                Tango::DevEncoded& w_buffer = buffer[1];
+                bopy::str w_encoded_format(w_buffer.encoded_format);
 
-    template<>
-    inline void _update_scalar_values<Tango::DEV_ENCODED>(Tango::DeviceAttribute &self, object py_value)
-    {
-        Tango::DevVarEncodedArray* value;
-        self >> value;
-        std::auto_ptr<Tango::DevVarEncodedArray> guard_value(value);
-        
-        Tango::DevEncoded* buffer = value->get_buffer();
-        
-        using namespace boost::python;
-        str r_encoded_format(buffer[0].encoded_format);
+                Tango::DevVarCharArray& w_encoded_data_array = w_buffer.encoded_data;
+                char* w_ch_ptr = (char*) w_encoded_data_array.get_buffer();
+                PyObject* w_encoded_data_ptr = NULL;
+                    PyByteArray_FromStringAndSize(w_ch_ptr, w_encoded_data_array.length());
+                Py_ssize_t w_size = w_encoded_data_array.length();
+                if (read_only)
+                {
+                    w_encoded_data_ptr = PyBytes_FromStringAndSize(w_ch_ptr, w_size);
+                }
+                else
+                {
+                    w_encoded_data_ptr = PyByteArray_FromStringAndSize(w_ch_ptr, w_size);
+                }
+                bopy::object w_encoded_data = bopy::object(bopy::handle<>(w_encoded_data_ptr));
 
-        Tango::DevVarCharArray &encReadBuffer = buffer[0].encoded_data;
-        Py_ssize_t size = encReadBuffer.length();
-        CORBA::Octet *rawReadBuffer = encReadBuffer.get_buffer(1);
-        
-        PyObject *readArray = PyBuffer_FromReadWriteMemory(rawReadBuffer, size);
-        
-        object r_encoded_data = object(handle<>(readArray));
-        
-        py_value.attr(value_attr_name) = make_tuple(r_encoded_format, r_encoded_data);
-
-        if (self.get_written_dim_x() > 0) {
-            bool is_write_type = self.get_written_dim_x() && (value->length() < 2);
-            if (is_write_type) {
-                object w_encoded_format(r_encoded_format);
-                object w_encoded_data(r_encoded_data);
-                py_value.attr(w_value_attr_name) = make_tuple(w_encoded_format, w_encoded_data);
-            } else {
-                str w_encoded_format(buffer[1].encoded_format);
-                
-                Tango::DevVarCharArray &encWriteBuffer = buffer[1].encoded_data;
-                int size = encWriteBuffer.length();
-                CORBA::Octet *rawWriteBuffer = encWriteBuffer.get_buffer(1);
-                
-                PyObject *writeArray = PyBuffer_FromReadWriteMemory(rawWriteBuffer, size);
-                
-                object w_encoded_data = object(handle<>(writeArray));
-                
-                py_value.attr(w_value_attr_name) = make_tuple(w_encoded_format, w_encoded_data);
+                py_value.attr(value_attr_name) =
+                    bopy::make_tuple(w_encoded_format, w_encoded_data);
             }
-        } else {
-            py_value.attr(w_value_attr_name) = object();
+        }
+        else
+        {
+            py_value.attr(w_value_attr_name) = bopy::object();
         }
     }
 
-    template<>
-    inline void _update_scalar_values<Tango::DEV_STRING>(Tango::DeviceAttribute &self, object py_value)
-    {
-        if (self.get_written_dim_x() > 0) {
-            std::vector<std::string> val;
-            self.extract_read(val);
-            py_value.attr(value_attr_name) = object(val[0]);
-            self.extract_set(val);
-            py_value.attr(w_value_attr_name) = object(val[0]);
-        } else {
-            std::string rvalue;
-            self >> rvalue;
-            py_value.attr(value_attr_name) = object(rvalue);
-            py_value.attr(w_value_attr_name) = object();
-        }
-    }
-
-    template<long tangoTypeConst>
-    static inline void _update_array_values_as_lists(Tango::DeviceAttribute &self, bool isImage, object py_value)
+    template<long tangoTypeConst> static inline void
+    _update_value_as_string(Tango::DeviceAttribute &self,
+                            bopy::object py_value)
     {
         typedef typename TANGO_const2type(tangoTypeConst) TangoScalarType;
         typedef typename TANGO_const2arraytype(tangoTypeConst) TangoArrayType;
 
         // Extract the actual data from Tango::DeviceAttribute (self)
         TangoArrayType* value_ptr = 0;
-        try {
-            self >> value_ptr;
-        } catch (Tango::DevFailed &e ) {
-            if (strcmp(e.errors[0].reason.in(),"API_EmptyDeviceAttribute") != 0)
-                throw;
+        EXTRACT_VALUE(self, value_ptr)
+        unique_pointer<TangoArrayType> guard_value_ptr(value_ptr);
+
+        if (value_ptr == 0)
+        {
+            py_value.attr(value_attr_name) = bopy::str();
+            py_value.attr(w_value_attr_name) = bopy::object();
+            return;
         }
-        std::auto_ptr<TangoArrayType> guard_value_ptr(value_ptr);
+
+        TangoScalarType* buffer = value_ptr->get_buffer();
+
+        const char* ch_ptr = reinterpret_cast<char *>(buffer);
+        size_t nb_bytes = value_ptr->length() * sizeof(TangoScalarType);
+
+        py_value.attr(value_attr_name) = bopy::str(ch_ptr, (size_t)nb_bytes);
+        py_value.attr(w_value_attr_name) = bopy::object();
+    }
+
+    template<> inline void
+    _update_value_as_string<Tango::DEV_ENCODED>(Tango::DeviceAttribute &self,
+                                                bopy::object py_value)
+    {
+        Tango::DevVarEncodedArray* value_ptr;
+        EXTRACT_VALUE(self, value_ptr)
+        unique_pointer<Tango::DevVarEncodedArray> guard(value_ptr);
+        
+        Tango::DevEncoded* buffer = value_ptr->get_buffer();
+
+        Tango::DevEncoded& r_buffer = buffer[0];
+        bopy::str r_encoded_format(r_buffer.encoded_format);
+
+        Tango::DevVarCharArray& r_encoded_data_array = r_buffer.encoded_data;
+        char* r_ch_ptr = (char*)r_encoded_data_array.get_buffer();
+        bopy::str r_encoded_data(r_ch_ptr, r_encoded_data_array.length());
+        
+        py_value.attr(value_attr_name) = 
+            bopy::make_tuple(r_encoded_format, r_encoded_data);
+
+        if (self.get_written_dim_x() > 0)
+        {
+            bool is_write_type = self.get_written_dim_x() && (value_ptr->length() < 2);
+            if (is_write_type)
+            {
+                bopy::object w_encoded_format(r_encoded_format);
+                bopy::object w_encoded_data(r_encoded_data);
+                py_value.attr(w_value_attr_name) =
+                    bopy::make_tuple(w_encoded_format, w_encoded_data);
+            }
+            else
+            {
+                Tango::DevEncoded& w_buffer = buffer[1];
+                bopy::str w_encoded_format(w_buffer.encoded_format);
+
+                Tango::DevVarCharArray& w_encoded_data_array = w_buffer.encoded_data;
+                char* w_ch_ptr = (char*)w_encoded_data_array.get_buffer();
+                bopy::str w_encoded_data(w_ch_ptr, w_encoded_data_array.length());
+                py_value.attr(w_value_attr_name) = 
+                    bopy::make_tuple(w_encoded_format, w_encoded_data);
+            }
+        }
+        else
+        {
+            py_value.attr(w_value_attr_name) = bopy::object();
+        }
+    }
+
+    template<long tangoTypeConst> static inline void
+    _update_scalar_values(Tango::DeviceAttribute &self, bopy::object py_value)
+    {
+        typedef typename TANGO_const2type(tangoTypeConst) TangoScalarType;
+        
+        if (self.get_written_dim_x() > 0)
+        {
+            std::vector<TangoScalarType> val;
+            self.extract_read(val);
+            // In the following lines, the cast is absolutely necessary because
+            // vector<TangoScalarType> may not be a vector<TangoScalarType> at
+            // compile time. For example, for vector<DevBoolean>, the compiler
+            // may create a std::_Bit_reference type.
+            py_value.attr(value_attr_name) = bopy::object((TangoScalarType)val[0]);
+            self.extract_set(val);
+            py_value.attr(w_value_attr_name) = bopy::object((TangoScalarType)val[0]);
+        }
+        else
+        {
+            TangoScalarType rvalue;
+            EXTRACT_VALUE(self, rvalue)
+            py_value.attr(value_attr_name) = bopy::object(rvalue);
+            py_value.attr(w_value_attr_name) = bopy::object();
+        }
+    }
+
+    template<> inline void 
+    _update_scalar_values<Tango::DEV_ENCODED>(Tango::DeviceAttribute &self,
+                                              bopy::object py_value)
+    {
+        _update_value_as_string<Tango::DEV_ENCODED>(self, py_value);
+    }
+
+    template<> inline void
+    _update_scalar_values<Tango::DEV_STRING>(Tango::DeviceAttribute &self,
+                                             bopy::object py_value)
+    {
+        if (self.get_written_dim_x() > 0)
+        {
+            std::vector<std::string> r_val, w_val;
+            self.extract_read(r_val);
+            py_value.attr(value_attr_name) = object(r_val[0]);
+            self.extract_set(w_val);
+            py_value.attr(w_value_attr_name) = object(w_val[0]);
+        }
+        else
+        {
+            std::string rvalue;
+            EXTRACT_VALUE(self, rvalue)
+            py_value.attr(value_attr_name) = object(rvalue);
+            py_value.attr(w_value_attr_name) = object();
+        }
+    }
+
+    template<long tangoTypeConst> static inline void
+    _update_array_values_as_lists(Tango::DeviceAttribute &self, bool isImage,
+                                  bopy::object py_value)
+    {
+        typedef typename TANGO_const2type(tangoTypeConst) TangoScalarType;
+        typedef typename TANGO_const2arraytype(tangoTypeConst) TangoArrayType;
+
+        // Extract the actual data from Tango::DeviceAttribute (self)
+        TangoArrayType* value_ptr = 0;
+        EXTRACT_VALUE(self, value_ptr)
+        unique_pointer<TangoArrayType> guard_value_ptr(value_ptr);
 
         if (value_ptr == 0) {
             // Empty device attribute
-            py_value.attr(value_attr_name) = boost::python::list();
+            py_value.attr(value_attr_name) = bopy::list();
             py_value.attr(w_value_attr_name) = object();
             return;
         }
@@ -281,14 +377,14 @@ namespace PyDeviceAttribute {
                 py_value.attr(w_value_attr_name) = py_value.attr(value_attr_name);
                 continue;
             }
-            boost::python::list result;
+            bopy::list result;
             
             if (isImage) {
                 const int dim_x = it? self.get_dim_x() : self.get_written_dim_x();
                 const int dim_y = it? self.get_dim_y() : self.get_written_dim_y();
                 
                 for (int y=0; y < dim_y; ++y) {
-                    boost::python::list row;
+                    bopy::list row;
                     for (int x=0; x < dim_x; ++x)
                         row.append(python_tangocpp<tangoTypeConst>::to_python(buffer[offset + x + y*dim_x]));
                     result.append(row);
@@ -304,67 +400,21 @@ namespace PyDeviceAttribute {
         }
     }
 
-    template<long tangoTypeConst>
-    static inline void _update_array_values_as_pytango3(Tango::DeviceAttribute &self, bool isImage, object py_value)
+    template<long tangoTypeConst> static void
+    _update_array_values_as_tuples(Tango::DeviceAttribute &self, bool isImage,
+                                   bopy::object py_value)
     {
         typedef typename TANGO_const2type(tangoTypeConst) TangoScalarType;
         typedef typename TANGO_const2arraytype(tangoTypeConst) TangoArrayType;
 
         // Extract the actual data from Tango::DeviceAttribute (self)
         TangoArrayType* value_ptr = 0;
-        try {
-            self >> value_ptr;
-        } catch (Tango::DevFailed &e ) {
-            if (strcmp(e.errors[0].reason.in(),"API_EmptyDeviceAttribute") != 0)
-                throw;
-        }
-        std::auto_ptr<TangoArrayType> guard_value_ptr(value_ptr);
+        EXTRACT_VALUE(self, value_ptr)
+        unique_pointer<TangoArrayType> guard_value_ptr(value_ptr);
 
         if (value_ptr == 0) {
             // Empty device attribute
-            py_value.attr(value_attr_name) = boost::python::list();
-            py_value.attr(w_value_attr_name) = object();
-            return;
-        }
-
-        TangoScalarType* buffer = value_ptr->get_buffer();
-
-        long sz = value_ptr->length();
-        boost::python::list res;
-        for (long x =0; x<sz; ++x) {
-            res.append(python_tangocpp<tangoTypeConst>::to_python(buffer[x]));
-        }
-
-        py_value.attr(value_attr_name) = res;
-        py_value.attr(w_value_attr_name) = object();
-    }
-
-    template<>
-    inline void _update_array_values_as_pytango3<Tango::DEV_ENCODED>(Tango::DeviceAttribute &self, bool isImage, object py_value)
-    {
-        /// @todo DevEncoded didn't even exist in PyTango3...
-        _update_array_values_as_tuples<Tango::DEV_ENCODED>(self, isImage, py_value);
-    }
-
-    template<long tangoTypeConst>
-    void _update_array_values_as_tuples(Tango::DeviceAttribute &self, bool isImage, object py_value)
-    {
-        typedef typename TANGO_const2type(tangoTypeConst) TangoScalarType;
-        typedef typename TANGO_const2arraytype(tangoTypeConst) TangoArrayType;
-
-        // Extract the actual data from Tango::DeviceAttribute (self)
-        TangoArrayType* value_ptr = 0;
-        try {
-            self >> value_ptr;
-        } catch (Tango::DevFailed &e ) {
-            if (strcmp(e.errors[0].reason.in(),"API_EmptyDeviceAttribute") != 0)
-                throw;
-        }
-        std::auto_ptr<TangoArrayType> guard_value_ptr(value_ptr);
-
-        if (value_ptr == 0) {
-            // Empty device attribute
-            py_value.attr(value_attr_name) = boost::python::tuple();
+            py_value.attr(value_attr_name) = bopy::tuple();
             py_value.attr(w_value_attr_name) = object();
             return;
         }
@@ -398,13 +448,13 @@ namespace PyDeviceAttribute {
 
                 PyObject * result = PyTuple_New(dim_y);
                 if (!result)
-                    boost::python::throw_error_already_set();
+                    bopy::throw_error_already_set();
                 result_guard = object(handle<>(result));
 
                 for (int y=0; y < dim_y; ++y) {
                     PyObject * row = PyTuple_New(dim_x);
                     if (!row)
-                        boost::python::throw_error_already_set();
+                        bopy::throw_error_already_set();
                     object row_guard = object(handle<>(row));
                     for (int x=0; x < dim_x; ++x) {
                         object el = python_tangocpp<tangoTypeConst>::to_python(buffer[offset + x + y*dim_x]);
@@ -420,7 +470,7 @@ namespace PyDeviceAttribute {
 
                 PyObject * result = PyTuple_New(dim_x);
                 if (!result)
-                    boost::python::throw_error_already_set();
+                    bopy::throw_error_already_set();
                 result_guard = object(handle<>(result));
 
                 for (int x=0; x < dim_x; ++x) {
@@ -434,13 +484,16 @@ namespace PyDeviceAttribute {
         }
     }
 
-    void update_values(Tango::DeviceAttribute &self, object& py_value, PyTango::ExtractAs extract_as/*=ExtractAsNumpy*/)
+    void
+    update_values(Tango::DeviceAttribute &self, bopy::object& py_value,
+                  PyTango::ExtractAs extract_as/*=ExtractAsNumpy*/)
     {
         // We do not want is_empty to launch an exception!!
         self.reset_exceptions(Tango::DeviceAttribute::isempty_flag);
+        
         // self.get_type() already does self.is_empty()
         const int data_type = self.get_type();
-        const bool is_empty = data_type < 0; /*self.is_empty()*/
+        const bool is_empty = data_type < 0;
         const bool has_failed = self.has_failed();
         Tango::AttrDataFormat data_format = self.get_data_format();
 
@@ -458,23 +511,40 @@ namespace PyDeviceAttribute {
         bool is_image = false;
         switch (data_format) {
             case Tango::SCALAR:
-                switch (extract_as)
+                if (data_type == Tango::DEV_ENCODED)
                 {
-                    default:
-                    case PyTango::ExtractAsNumpy:
-                    case PyTango::ExtractAsTuple:
-                    case PyTango::ExtractAsList:
-                        TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE(data_type, _update_scalar_values, self, py_value);
-                        break;
-                    case PyTango::ExtractAsString:
-                        TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE(data_type, _update_value_as_string, self, py_value);
-                        break;
-                    case PyTango::ExtractAsPyTango3:
-                        TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE(data_type, _update_scalar_values, self, py_value);
-                        py_value.attr("w_scalar_value") = py_value.attr(w_value_attr_name);
-                        break;
-                    case PyTango::ExtractAsNothing:
-                        break;
+                    switch (extract_as)
+                    {
+                        default:
+                        case PyTango::ExtractAsNumpy:
+                        case PyTango::ExtractAsTuple:
+                        case PyTango::ExtractAsList:
+                            TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE_ID(data_type,
+                                _update_scalar_values, self, py_value);
+                            break;
+                        case PyTango::ExtractAsBytes:
+                            TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE_ID(data_type,
+                                _update_value_as_bin, self, py_value, true);
+                            break;
+                        case PyTango::ExtractAsByteArray:
+                            TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE_ID(data_type,
+                                _update_value_as_bin, self, py_value, false);
+                            break;
+                        case PyTango::ExtractAsString:
+                            TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE_ID(data_type,
+                                _update_value_as_string, self, py_value);
+                            break;
+                        case PyTango::ExtractAsNothing:
+                            break;
+                    }
+                }
+                else
+                {
+                    if (extract_as != PyTango::ExtractAsNothing)
+                    {
+                        TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE_ID(data_type,
+                            _update_scalar_values, self, py_value);
+                    }
                 }
                 break;
             case Tango::IMAGE:
@@ -485,25 +555,29 @@ namespace PyDeviceAttribute {
                     default:
                     case PyTango::ExtractAsNumpy:
 #                   ifndef DISABLE_PYTANGO_NUMPY
-                        TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE(data_type, _update_array_values,
-                                           self, is_image, py_value);
+                        TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE_ID(data_type,
+                            _update_array_values, self, is_image, py_value);
                         break;
 #                   endif
                     case PyTango::ExtractAsTuple:
-                        TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE(data_type, _update_array_values_as_tuples,
-                                           self, is_image, py_value);
+                        TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE_ID(data_type,
+                            _update_array_values_as_tuples, self, is_image, py_value);
                         break;
                     case PyTango::ExtractAsList:
-                        TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE(data_type, _update_array_values_as_lists,
-                                           self, is_image, py_value);
+                        TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE_ID(data_type,
+                            _update_array_values_as_lists, self, is_image, py_value);
+                        break;
+                    case PyTango::ExtractAsBytes:
+                        TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE_ID(data_type,
+                            _update_value_as_bin, self, py_value, true);
+                        break;
+                    case PyTango::ExtractAsByteArray:
+                        TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE_ID(data_type,
+                            _update_value_as_bin, self, py_value, false);
                         break;
                     case PyTango::ExtractAsString:
-                        TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE(data_type, _update_value_as_string,
-                                           self, py_value);
-                        break;
-                    case PyTango::ExtractAsPyTango3:
-                        TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE(data_type, _update_array_values_as_pytango3,
-                                           self, is_image, py_value);
+                        TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE_ID(data_type,
+                            _update_value_as_string, self, py_value);
                         break;
                     case PyTango::ExtractAsNothing:
                         break;
@@ -516,8 +590,9 @@ namespace PyDeviceAttribute {
         }
     }
 
-    template<long tangoTypeConst>
-    static inline void _fill_list_attribute(Tango::DeviceAttribute & dev_attr, const bool isImage, const object & py_value)
+    template<long tangoTypeConst> static inline void
+    _fill_list_attribute(Tango::DeviceAttribute & dev_attr, const bool isImage,
+                         const bopy::object & py_value)
     {
         typedef typename TANGO_const2type(tangoTypeConst) TangoScalarType;
         typedef typename TANGO_const2arraytype(tangoTypeConst) TangoArrayType;
@@ -536,7 +611,7 @@ namespace PyDeviceAttribute {
         }
 
         // -- Allocate memory
-        std::auto_ptr<TangoArrayType> value;
+        unique_pointer<TangoArrayType> value;
         TangoScalarType* buffer = TangoArrayType::allocbuf(nelems);
         try {
             value.reset(new TangoArrayType(nelems, nelems, buffer, true));
@@ -568,26 +643,31 @@ namespace PyDeviceAttribute {
         value.release(); // Do not delete value, it is handled by dev_attr now!
     }
 
-    template<>
-    inline void _fill_list_attribute<Tango::DEV_ENCODED>(Tango::DeviceAttribute & dev_attr, const bool isImage, const object & py_value)
+    template<> inline void
+    _fill_list_attribute<Tango::DEV_ENCODED>(Tango::DeviceAttribute & dev_attr,
+                                             const bool isImage,
+                                             const bopy::object & py_value)
     {
         /// @todo really? This is really not gonna happen?
         // Unsupported
         assert(false);
     }
 
-    static inline object undefined_attribute(Tango::DeviceAttribute* self)
+    static inline bopy::object
+    undefined_attribute(Tango::DeviceAttribute* self)
     {
         return object(); // None
     }
 
-    void reset_values(Tango::DeviceAttribute & self, int data_type, Tango::AttrDataFormat data_format, object py_value)
+    void
+    reset_values(Tango::DeviceAttribute & self, int data_type,
+                 Tango::AttrDataFormat data_format, bopy::object py_value)
     {
         bool isImage = false;
         switch(data_format)
         {
             case Tango::SCALAR:
-                TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE( data_type, _fill_scalar_attribute, self, py_value );
+                TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE_ID( data_type, _fill_scalar_attribute, self, py_value );
                 break;
             case Tango::IMAGE:
                 isImage = true;
@@ -603,14 +683,14 @@ namespace PyDeviceAttribute {
                 // standard types and C++ are.
 #               ifdef DISABLE_PYTANGO_NUMPY
                 {
-                    TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE( data_type, _fill_list_attribute, self, isImage, py_value );
+                    TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE_ID( data_type, _fill_list_attribute, self, isImage, py_value );
                 }
 #               else
                 {
                     if (PyArray_Check(py_value.ptr()))
-                        TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE( data_type, _fill_numpy_attribute, self, isImage, py_value );
+                        TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE_ID( data_type, _fill_numpy_attribute, self, isImage, py_value );
                     else
-                        TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE( data_type, _fill_list_attribute, self, isImage, py_value );
+                        TANGO_CALL_ON_ATTRIBUTE_DATA_TYPE_ID( data_type, _fill_list_attribute, self, isImage, py_value );
                     break;
                 }
 #               endif
@@ -619,13 +699,17 @@ namespace PyDeviceAttribute {
         }
     }
 
-    void reset(Tango::DeviceAttribute & self, const Tango::AttributeInfo &attr_info, object py_value)
+    void
+    reset(Tango::DeviceAttribute & self, const Tango::AttributeInfo &attr_info,
+          bopy::object py_value)
     {
         self.set_name(const_cast<std::string&>(attr_info.name));
         reset_values(self, attr_info.data_type, attr_info.data_format, py_value);
     }
 
-    void reset(Tango::DeviceAttribute & self, const std::string &attr_name, Tango::DeviceProxy &dev_proxy, object py_value)
+    void
+    reset(Tango::DeviceAttribute & self, const std::string &attr_name,
+          Tango::DeviceProxy &dev_proxy, bopy::object py_value)
     {
         Tango::AttributeInfoEx attr_info;
         {
@@ -670,5 +754,7 @@ void export_device_attribute()
             return_internal_reference<>())
         .def("get_err_stack", &Tango::DeviceAttribute::get_err_stack,
             return_value_policy<copy_const_reference>())
+        .def("set_w_dim_x", &Tango::DeviceAttribute::set_w_dim_x)
+        .def("set_w_dim_y", &Tango::DeviceAttribute::set_w_dim_y)
     ;
 }
