@@ -21,20 +21,12 @@
    
 *******************************************************************************/
 
-#include <boost/python.hpp>
-#include <boost/python/return_value_policy.hpp>
-#include <tango.h>
-#include <string>
-#include <memory>
-
+#include "precompiled_header.hpp"
 #include "device_attribute.h"
 #include "callback.h"
-
 #include "defs.h"
 #include "pytgutils.h"
 
-using namespace boost::python;
-
 extern const char *param_must_be_seq;
 extern const char *unreachable_code;
 extern const char *non_string_seq;
@@ -44,12 +36,13 @@ BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(unlock_overloads, Tango::DeviceProxy::unl
 
 namespace PyDeviceProxy
 {
-    struct PickleSuite : pickle_suite
+    struct PickleSuite : bopy::pickle_suite
     {
-        static tuple getinitargs(Tango::DeviceProxy& self)
+        static bopy::tuple getinitargs(Tango::DeviceProxy& self)
         {
-            std::string ret = self.get_db_host() + ":" + self.get_db_port() + "/" + self.dev_name();
-            return make_tuple(ret);
+            std::string ret = self.get_db_host() + ":" + self.get_db_port() +
+                              "/" + self.dev_name();
+            return bopy::make_tuple(ret);
         }
     };
     
@@ -71,22 +64,23 @@ namespace PyDeviceProxy
         return self.ping();
     }
 
-    static inline void pylist_to_devattrs(Tango::DeviceProxy& self, object &py_list, std::vector<Tango::DeviceAttribute> &dev_attrs)
+    static inline void pylist_to_devattrs(Tango::DeviceProxy& self,
+        bopy::object &py_list, std::vector<Tango::DeviceAttribute> &dev_attrs)
     {
         std::vector<std::string> attr_names;
-        std::vector<object> py_values;
+        std::vector<bopy::object> py_values;
         long size = len(py_list);
 
         // Fill attr_names and py_values
         for (long n = 0; n < size; ++n) {
-            object tup = py_list[n];
-            std::string attr_name = extract<std::string>(tup[0]);
+            bopy::object tup = py_list[n];
+            std::string attr_name = bopy::extract<std::string>(tup[0]);
             attr_names.push_back(attr_name);
             py_values.push_back(tup[1]);
         }
 
         // Get attr_info for all the attr_names
-        std::auto_ptr<Tango::AttributeInfoListEx> attr_infos;
+        unique_pointer<Tango::AttributeInfoListEx> attr_infos;
         {
             AutoPythonAllowThreads guard;
             attr_infos.reset(self.get_attribute_config_ex(attr_names));
@@ -99,7 +93,7 @@ namespace PyDeviceProxy
         }
     }
 
-    static inline object read_attribute(Tango::DeviceProxy& self, const string & attr_name, PyTango::ExtractAs extract_as)
+    static inline bopy::object read_attribute(Tango::DeviceProxy& self, const string & attr_name, PyTango::ExtractAs extract_as)
     {
         // Even if there's an exception in convert_to_python, the
         // DeviceAttribute will be deleted there, so we don't need to worry.
@@ -112,7 +106,7 @@ namespace PyDeviceProxy
         return PyDeviceAttribute::convert_to_python(dev_attr, self, extract_as);
     }
 
-    static inline object read_attributes(Tango::DeviceProxy& self, object py_attr_names, PyTango::ExtractAs extract_as)
+    static inline bopy::object read_attributes(Tango::DeviceProxy& self, bopy::object py_attr_names, PyTango::ExtractAs extract_as)
     {
         CSequenceFromPython<StdStringVector> attr_names(py_attr_names);
 
@@ -125,16 +119,15 @@ namespace PyDeviceProxy
         return PyDeviceAttribute::convert_to_python(dev_attr_vec, self, extract_as);
     }
 
-    static inline void write_attribute(Tango::DeviceProxy& self, const Tango::AttributeInfo & attr_info, object py_value)
+    static inline void write_attribute(Tango::DeviceProxy& self, const Tango::AttributeInfo & attr_info, bopy::object py_value)
     {
-
         Tango::DeviceAttribute da;
         PyDeviceAttribute::reset(da, attr_info, py_value);
         AutoPythonAllowThreads guard;
         self.write_attribute(da);
     }
 
-    static inline void write_attribute(Tango::DeviceProxy& self, const string & attr_name, object py_value)
+    static inline void write_attribute(Tango::DeviceProxy& self, const string & attr_name, bopy::object py_value)
     {
         Tango::DeviceAttribute dev_attr;
         PyDeviceAttribute::reset(dev_attr, attr_name, self, py_value);
@@ -144,7 +137,7 @@ namespace PyDeviceProxy
         }
     }
 
-    static inline void write_attributes(Tango::DeviceProxy& self, object py_list)
+    static inline void write_attributes(Tango::DeviceProxy& self, bopy::object py_list)
     {
         std::vector<Tango::DeviceAttribute> dev_attrs;
         pylist_to_devattrs(self, py_list, dev_attrs);
@@ -153,10 +146,10 @@ namespace PyDeviceProxy
         self.write_attributes(dev_attrs);
     }
 
-    static inline object write_read_attribute(Tango::DeviceProxy& self, const string & attr_name, object py_value, PyTango::ExtractAs extract_as)
+    static inline bopy::object write_read_attribute(Tango::DeviceProxy& self, const string & attr_name, bopy::object py_value, PyTango::ExtractAs extract_as)
     {
         Tango::DeviceAttribute w_dev_attr;
-        std::auto_ptr<Tango::DeviceAttribute> r_dev_attr(0);
+        unique_pointer<Tango::DeviceAttribute> r_dev_attr;
 
         // Prepare dev_attr structure
         PyDeviceAttribute::reset(w_dev_attr, attr_name, self, py_value);
@@ -171,18 +164,30 @@ namespace PyDeviceProxy
         return PyDeviceAttribute::convert_to_python(r_dev_attr.release(), self, extract_as);
     }
 
-
-    static inline vector<Tango::DeviceDataHistory>*
-            command_history(Tango::DeviceProxy& self, const std::string & cmd_name, int depth)
+    static inline bopy::object
+    command_history(Tango::DeviceProxy& self, const std::string& cmd_name, int depth)
     {
-        AutoPythonAllowThreads guard;
-        return self.command_history(const_cast<std::string&>(cmd_name), depth);
+        std::vector<Tango::DeviceDataHistory>* device_data_hist = NULL;
+        bopy::list ret;
+        {
+            AutoPythonAllowThreads guard;
+            device_data_hist =
+                self.command_history(const_cast<std::string&>(cmd_name), depth);
+        }
+        vector<Tango::DeviceDataHistory>::iterator it = device_data_hist->begin();
+        for(;it != device_data_hist->end(); ++it)
+        {
+            Tango::DeviceDataHistory& hist = *it;
+            ret.append(hist);
+        }
+        delete device_data_hist;
+        return ret;
     }
 
-    static inline object
+    static inline bopy::object
             attribute_history(Tango::DeviceProxy& self, const std::string & attr_name, int depth, PyTango::ExtractAs extract_as)
     {
-        std::auto_ptr< vector<Tango::DeviceAttributeHistory> > att_hist;
+        unique_pointer< vector<Tango::DeviceAttributeHistory> > att_hist;
         {
             AutoPythonAllowThreads guard;
             att_hist.reset(self.attribute_history(const_cast<std::string&>(attr_name), depth));
@@ -191,7 +196,7 @@ namespace PyDeviceProxy
     }
 
 
-    static inline long read_attributes_asynch(Tango::DeviceProxy& self, object py_attr_names)
+    static inline long read_attributes_asynch(Tango::DeviceProxy& self, bopy::object py_attr_names)
     {
         CSequenceFromPython<StdStringVector> attr_names(py_attr_names);
 
@@ -201,7 +206,7 @@ namespace PyDeviceProxy
 
 
 
-    static inline object read_attributes_reply(Tango::DeviceProxy& self, long id, PyTango::ExtractAs extract_as)
+    static inline bopy::object read_attributes_reply(Tango::DeviceProxy& self, long id, PyTango::ExtractAs extract_as)
     {
         PyDeviceAttribute::AutoDevAttrVector dev_attr_vec;
         {
@@ -211,7 +216,7 @@ namespace PyDeviceProxy
         return PyDeviceAttribute::convert_to_python(dev_attr_vec, self, extract_as);
     }
 
-    static inline object read_attributes_reply(Tango::DeviceProxy& self, long id, long timeout, PyTango::ExtractAs extract_as)
+    static inline bopy::object read_attributes_reply(Tango::DeviceProxy& self, long id, long timeout, PyTango::ExtractAs extract_as)
     {
         PyDeviceAttribute::AutoDevAttrVector dev_attr_vec;
         {
@@ -221,7 +226,7 @@ namespace PyDeviceProxy
         return PyDeviceAttribute::convert_to_python(dev_attr_vec, self, extract_as);
     }
 
-    static inline long write_attributes_asynch(Tango::DeviceProxy& self, object py_list)
+    static inline long write_attributes_asynch(Tango::DeviceProxy& self, bopy::object py_list)
     {
         std::vector<Tango::DeviceAttribute> dev_attrs;
         pylist_to_devattrs(self, py_list, dev_attrs);
@@ -242,12 +247,12 @@ namespace PyDeviceProxy
         self.write_attributes_reply(id);
     }
 
-    static inline void read_attributes_asynch(object py_self, object py_attr_names, object py_cb, PyTango::ExtractAs extract_as)
+    static inline void read_attributes_asynch(bopy::object py_self, bopy::object py_attr_names, bopy::object py_cb, PyTango::ExtractAs extract_as)
     {
-        Tango::DeviceProxy* self = extract<Tango::DeviceProxy*>(py_self);
+        Tango::DeviceProxy* self = bopy::extract<Tango::DeviceProxy*>(py_self);
         CSequenceFromPython<StdStringVector> attr_names(py_attr_names);
 
-        PyCallBackAutoDie* cb = extract<PyCallBackAutoDie*>(py_cb);
+        PyCallBackAutoDie* cb = bopy::extract<PyCallBackAutoDie*>(py_cb);
         cb->set_autokill_references(py_cb, py_self);
         cb->set_extract_as(extract_as);
 
@@ -260,13 +265,13 @@ namespace PyDeviceProxy
         }
     }
 
-    static inline void write_attributes_asynch(object py_self, object py_list, object py_cb)
+    static inline void write_attributes_asynch(bopy::object py_self, bopy::object py_list, bopy::object py_cb)
     {
-        Tango::DeviceProxy* self = extract<Tango::DeviceProxy*>(py_self);
+        Tango::DeviceProxy* self = bopy::extract<Tango::DeviceProxy*>(py_self);
         std::vector<Tango::DeviceAttribute> dev_attrs;
         pylist_to_devattrs(*self, py_list, dev_attrs);
 
-        PyCallBackAutoDie* cb = extract<PyCallBackAutoDie*>(py_cb);
+        PyCallBackAutoDie* cb = bopy::extract<PyCallBackAutoDie*>(py_cb);
         cb->set_autokill_references(py_cb, py_self);
 
         try {
@@ -279,21 +284,21 @@ namespace PyDeviceProxy
     }
 
     static int subscribe_event(
-            object py_self,
+            bopy::object py_self,
             const string &attr_name,
             Tango::EventType event,
-            object py_cb_or_queuesize,
-            object &py_filters,
+            bopy::object py_cb_or_queuesize,
+            bopy::object &py_filters,
             bool stateless,
             PyTango::ExtractAs extract_as )
     {
-        Tango::DeviceProxy& self = extract<Tango::DeviceProxy&>(py_self);
+        Tango::DeviceProxy& self = bopy::extract<Tango::DeviceProxy&>(py_self);
         CSequenceFromPython<StdStringVector> filters(py_filters);
 
         PyCallBackPushEvent* cb = 0;
         int event_queue_size = 0;
-        if (extract<PyCallBackPushEvent&>(py_cb_or_queuesize).check()) {
-            cb = extract<PyCallBackPushEvent*>(py_cb_or_queuesize);
+        if (bopy::extract<PyCallBackPushEvent&>(py_cb_or_queuesize).check()) {
+            cb = bopy::extract<PyCallBackPushEvent*>(py_cb_or_queuesize);
 
             cb->set_device(py_self);
             cb->set_extract_as(extract_as);
@@ -301,9 +306,10 @@ namespace PyDeviceProxy
             AutoPythonAllowThreads guard;
             return self.subscribe_event(attr_name, event, cb, *filters, stateless);
         } else {
-            event_queue_size = extract<int>(py_cb_or_queuesize);
+            event_queue_size = bopy::extract<int>(py_cb_or_queuesize);
             AutoPythonAllowThreads guard;
-            return self.subscribe_event(attr_name, event, event_queue_size, *filters, stateless);
+            return self.subscribe_event(attr_name, event, event_queue_size,
+                                        *filters, stateless);
         }
     }
 
@@ -316,26 +322,26 @@ namespace PyDeviceProxy
     }
 
     template<typename ED, typename EDList>
-    static object
-    get_events__aux(object py_self, int event_id, PyTango::ExtractAs extract_as=PyTango::ExtractAsNumpy)
+    static bopy::object
+    get_events__aux(bopy::object py_self, int event_id,
+                    PyTango::ExtractAs extract_as=PyTango::ExtractAsNumpy)
     {
-        Tango::DeviceProxy &self = extract<Tango::DeviceProxy&>(py_self);
+        Tango::DeviceProxy &self = bopy::extract<Tango::DeviceProxy&>(py_self);
 
         EDList event_list;
         self.get_events(event_id, event_list);
 
-        boost::python::list r;
+        bopy::list r;
 
         for (size_t i=0; i < event_list.size(); ++i) {
             ED* event_data = event_list[i];
 
-            object py_ev(handle<>(
-                            to_python_indirect<
-                                ED*,
-                                detail::make_owning_holder>()(event_data)));
+            bopy::object py_ev(bopy::handle<>(
+                bopy::to_python_indirect<
+                    ED*, bopy::detail::make_owning_holder>()(event_data)));
 
             // EventDataList deletes EventData's on destructor, so once
-            // we are handling it somewhere else (as an object) we must
+            // we are handling it somewhere else (as an bopy::object) we must
             // unset the reference
             event_list[i] = 0;
 
@@ -347,9 +353,10 @@ namespace PyDeviceProxy
     }
 
     static void
-    get_events__callback(object py_self, int event_id, PyCallBackPushEvent *cb, PyTango::ExtractAs extract_as)
+    get_events__callback(bopy::object py_self, int event_id,
+                         PyCallBackPushEvent *cb, PyTango::ExtractAs extract_as)
     {
-        Tango::DeviceProxy& self = extract<Tango::DeviceProxy&>(py_self);
+        Tango::DeviceProxy& self = bopy::extract<Tango::DeviceProxy&>(py_self);
 
         cb->set_device(py_self);
         cb->set_extract_as(extract_as);
@@ -357,22 +364,22 @@ namespace PyDeviceProxy
         self.get_events(event_id, cb);
     }
 
-    static object
-    get_events__attr_conf(object py_self, int event_id)
+    static bopy::object
+    get_events__attr_conf(bopy::object py_self, int event_id)
     {
         return get_events__aux<Tango::AttrConfEventData, Tango::AttrConfEventDataList>
                                                 (py_self, event_id);
     }
 
-    static object
-    get_events__data(object py_self, int event_id, PyTango::ExtractAs extract_as)
+    static bopy::object
+    get_events__data(bopy::object py_self, int event_id, PyTango::ExtractAs extract_as)
     {
         return get_events__aux<Tango::EventData, Tango::EventDataList>
                                                 (py_self, event_id, extract_as);
     }
 
-    static object
-    get_events__data_ready(object py_self, int event_id)
+    static bopy::object
+    get_events__data_ready(bopy::object py_self, int event_id)
     {
         return get_events__aux<Tango::DataReadyEventData, Tango::DataReadyEventDataList>
                                                 (py_self, event_id);
@@ -391,15 +398,14 @@ void export_device_proxy()
     void (Tango::DeviceProxy::*delete_property_)(std::string &) =
         &Tango::DeviceProxy::delete_property;
 
-    class_<Tango::DeviceProxy, bases<Tango::Connection> > DeviceProxy(
-        "DeviceProxy",
-        init<>())
+    bopy::class_<Tango::DeviceProxy, bopy::bases<Tango::Connection> >
+        DeviceProxy("DeviceProxy", bopy::init<>())
     ;
 
     DeviceProxy
-        .def(init<const char *>())
-        .def(init<const char *, bool>())
-        .def(init<const Tango::DeviceProxy &>())
+        .def(bopy::init<const char *>())
+        .def(bopy::init<const char *, bool>())
+        .def(bopy::init<const Tango::DeviceProxy &>())
 
         //
         // Pickle
@@ -413,10 +419,10 @@ void export_device_proxy()
         
         .def("info", &Tango::DeviceProxy::info,
             ( arg_("self") ),
-            return_internal_reference<1>() )
+            bopy::return_internal_reference<1>() )
 
         .def("get_device_db", &Tango::DeviceProxy::get_device_db,
-            return_value_policy<reference_existing_object>())
+            bopy::return_value_policy<bopy::reference_existing_object>())
 
         .def("status", &PyDeviceProxy::status,
             ( arg_("self") ) )
@@ -442,7 +448,7 @@ void export_device_proxy()
 
         .def("black_box", &Tango::DeviceProxy::black_box,
             ( arg_("self"), arg_("n") ),
-            return_value_policy<manage_new_object>() )
+            bopy::return_value_policy<bopy::manage_new_object>() )
 
         //
         // device methods
@@ -453,7 +459,7 @@ void export_device_proxy()
 
         .def("command_list_query", &Tango::DeviceProxy::command_list_query,
             ( arg_("self") ),
-            return_value_policy<manage_new_object>() )
+            bopy::return_value_policy<bopy::manage_new_object>() )
 
         .def("import_info", &Tango::DeviceProxy::import_info,
             ( arg_("self") ) )
@@ -500,13 +506,13 @@ void export_device_proxy()
 
         .def("get_attribute_list", &Tango::DeviceProxy::get_attribute_list,
             ( arg_("self") ),
-            return_value_policy<manage_new_object>() )
+            bopy::return_value_policy<bopy::manage_new_object>() )
 
         .def("_get_attribute_config",
             (Tango::AttributeInfoList* (Tango::DeviceProxy::*)(StdStringVector &))
             &Tango::DeviceProxy::get_attribute_config,
             ( arg_("self"), arg_("attr_names") ),
-            return_value_policy<manage_new_object>() )
+            bopy::return_value_policy<bopy::manage_new_object>() )
 
         .def("_get_attribute_config",
             (Tango::AttributeInfoEx (Tango::DeviceProxy::*)(const std::string&))
@@ -516,19 +522,19 @@ void export_device_proxy()
         .def("_get_attribute_config_ex",
             &Tango::DeviceProxy::get_attribute_config_ex,
             ( arg_("self"), arg_("attr_names") ),
-            return_value_policy<manage_new_object>() )
+            bopy::return_value_policy<bopy::manage_new_object>() )
 
         .def("attribute_query", &Tango::DeviceProxy::attribute_query,
             ( arg_("self"), arg_("attr_name") ) )
 
         .def("attribute_list_query", &Tango::DeviceProxy::attribute_list_query,
             ( arg_("self") ),
-            return_value_policy<manage_new_object>() )
+            bopy::return_value_policy<bopy::manage_new_object>() )
 
         .def("attribute_list_query_ex",
             &Tango::DeviceProxy::attribute_list_query_ex,
             ( arg_("self") ),
-            return_value_policy<manage_new_object>() )
+            bopy::return_value_policy<bopy::manage_new_object>() )
 
         .def("_set_attribute_config",
             (void (Tango::DeviceProxy::*)(Tango::AttributeInfoList &))
@@ -549,12 +555,12 @@ void export_device_proxy()
             ( arg_("self"), arg_("attr_names"), arg_("extract_as")=PyTango::ExtractAsNumpy ) )
 
         .def("write_attribute",
-            (void (*)(Tango::DeviceProxy&, const string &, object ))
+            (void (*)(Tango::DeviceProxy&, const string &, bopy::object ))
             &PyDeviceProxy::write_attribute,
             ( arg_("self"), arg_("attr_name"), arg_("value") ) )
 
         .def("write_attribute",
-            (void (*)(Tango::DeviceProxy&, const Tango::AttributeInfo &, object ))
+            (void (*)(Tango::DeviceProxy&, const Tango::AttributeInfo &, bopy::object ))
             &PyDeviceProxy::write_attribute,
             ( arg_("self"), arg_("attr_info"), arg_("value") ) )
 
@@ -571,8 +577,7 @@ void export_device_proxy()
         //
 
         .def("command_history", &PyDeviceProxy::command_history,
-            (arg_("self"), arg_("cmd_name"), arg_("depth")),
-            return_value_policy<manage_new_object>() )
+            (arg_("self"), arg_("cmd_name"), arg_("depth")))
 
         .def("attribute_history", &PyDeviceProxy::attribute_history,
             (   arg_("self"),
@@ -586,7 +591,7 @@ void export_device_proxy()
 
         .def("polling_status", &Tango::DeviceProxy::polling_status,
             ( arg_("self") ),
-            return_value_policy<manage_new_object>() )
+            bopy::return_value_policy<bopy::manage_new_object>() )
 
         .def("poll_command",
             (void (Tango::DeviceProxy::*)(const char *, int)) &Tango::DeviceProxy::poll_command,
@@ -624,17 +629,17 @@ void export_device_proxy()
         // Asynchronous methods
         //
         .def("__read_attributes_asynch",
-            (long (*) (Tango::DeviceProxy &, object))
+            (long (*) (Tango::DeviceProxy &, bopy::object))
             &PyDeviceProxy::read_attributes_asynch,
             ( arg_("self"), arg_("attr_names") ) )
 
         .def("read_attributes_reply",
-            (object (*) (Tango::DeviceProxy &, long, PyTango::ExtractAs))
+            (bopy::object (*) (Tango::DeviceProxy &, long, PyTango::ExtractAs))
             &PyDeviceProxy::read_attributes_reply,
             ( arg_("self"), arg_("id"), arg_("extract_as")=PyTango::ExtractAsNumpy ) )
 
         .def("read_attributes_reply",
-            (object (*) (Tango::DeviceProxy &, long, long, PyTango::ExtractAs))
+            (bopy::object (*) (Tango::DeviceProxy &, long, long, PyTango::ExtractAs))
             &PyDeviceProxy::read_attributes_reply,
             ( arg_("self"), arg_("id"), arg_("timeout"), arg_("extract_as")=PyTango::ExtractAsNumpy ) )
 
@@ -643,7 +648,7 @@ void export_device_proxy()
             ( arg_("self"), arg_("req_type") ) )
 
         .def("__write_attributes_asynch",
-            (long (*) (Tango::DeviceProxy &, object))
+            (long (*) (Tango::DeviceProxy &, bopy::object))
             &PyDeviceProxy::write_attributes_asynch,
             ( arg_("self"), arg_("values") ) )
 
@@ -658,12 +663,12 @@ void export_device_proxy()
             ( arg_("self"), arg_("id"), arg_("timeout") ) )
 
         .def("__read_attributes_asynch",
-            (void (*) (object, object, object, PyTango::ExtractAs))
+            (void (*) (bopy::object, bopy::object, bopy::object, PyTango::ExtractAs))
             &PyDeviceProxy::read_attributes_asynch,
             ( arg_("self"), arg_("attr_names"), arg_("callback"), arg_("extract_as")=PyTango::ExtractAsNumpy ) )
 
         .def("__write_attributes_asynch",
-            (void (*) (object, object, object))
+            (void (*) (bopy::object, bopy::object, bopy::object))
             &PyDeviceProxy::write_attributes_asynch,
             ( arg_("self"), arg_("values"), arg_("callback") ) )
         
@@ -697,7 +702,7 @@ void export_device_proxy()
                 arg_("attr_name"),
                 arg_("event"),
                 arg_("cb_or_queuesize"),
-                arg_("filters")=boost::python::list(),
+                arg_("filters")=bopy::list(),
                 arg_("stateless")=false,
                 arg_("extract_as")=PyTango::ExtractAsNumpy )
             )
@@ -748,6 +753,6 @@ void export_device_proxy()
             ( arg_("self"), arg_("lockinfo") ))
 
         /// This is to be used by the python layer of this api...
-        //.setattr("__subscribed_events", boost::python::dict())
+        //.setattr("__subscribed_events", bopy::dict())
         ;
 }
