@@ -4,27 +4,77 @@
 
    http://www.tango-controls.org/static/PyTango/latest/doc/html/index.html
 
-   (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
-  
-   This is free software; you can redistribute it and/or modify
+   Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
+   
+   PyTango is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
-   the Free Software Foundation; either version 3 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
-  
-   This software is distributed in the hope that it will be useful,
+   
+   PyTango is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
   
    You should have received a copy of the GNU Lesser General Public License
-   along with this program; if not, see <http://www.gnu.org/licenses/>.
+   along with PyTango.  If not, see <http://www.gnu.org/licenses/>.
    
 *******************************************************************************/
 
+#include "precompiled_header.hpp"
 #include "from_py.h"
 
 using namespace boost::python;
 
+char* obj_to_new_char(PyObject* obj_ptr)
+{
+    Tango::DevString ret = NULL;
+    if(PyUnicode_Check(obj_ptr))
+    {
+        PyObject* obj_bytes_ptr = PyUnicode_AsLatin1String(obj_ptr);
+        ret = CORBA::string_dup(PyBytes_AsString(obj_bytes_ptr));
+        Py_DECREF(obj_bytes_ptr);
+    }
+    else
+    {
+        ret = CORBA::string_dup(PyBytes_AsString(obj_ptr));
+    }
+    return ret;
+}
+
+char* obj_to_new_char(bopy::object obj)
+{
+    return obj_to_new_char(obj.ptr());
+}
+
+void obj_to_string(PyObject* obj_ptr, std::string& result)
+{
+    if(PyUnicode_Check(obj_ptr))
+    {
+        PyObject* obj_bytes_ptr = PyUnicode_AsLatin1String(obj_ptr);
+        result = PyBytes_AsString(obj_bytes_ptr);
+        Py_DECREF(obj_bytes_ptr);
+    }
+    else
+    {
+        result = PyBytes_AsString(obj_ptr);
+    }
+}
+
+void obj_to_string(bopy::object obj, std::string& result)
+{
+    return obj_to_string(obj.ptr(), result);
+}
+
+/// @bug Not a bug per se, but you should keep in mind: It returns a new
+/// string, so if you pass it to Tango with a release flag there will be
+/// no problems, but if you have to use it yourself then you must remember
+/// to delete[] it!
+Tango::DevString PyString_AsCorbaString(PyObject* obj_ptr)
+{
+    return obj_to_new_char(obj_ptr);
+}
+
 void convert2array(const boost::python::object &py_value, Tango::DevVarCharArray & result)
 {
     PyObject *py_value_ptr = py_value.ptr();
@@ -35,9 +85,9 @@ void convert2array(const boost::python::object &py_value, Tango::DevVarCharArray
 
     size_t size = boost::python::len(py_value);
     result.length(size);
-    if (PyString_Check(py_value_ptr))
+    if (PyBytes_Check(py_value_ptr))
     {
-        char *ch = PyString_AS_STRING(py_value_ptr);
+        char *ch = PyBytes_AS_STRING(py_value_ptr);
         for (size_t i=0; i < size; ++i) {
             result[i] = ch[i];
         }
@@ -59,9 +109,15 @@ void convert2array(const object &py_value, StdStringVector & result)
         raise_(PyExc_TypeError, param_must_be_seq);
     }
     
-    if (PyString_Check(py_value_ptr))
+    if (PyBytes_Check(py_value_ptr))
     {
-        result.push_back(PyString_AsString(py_value_ptr));
+        result.push_back(PyBytes_AS_STRING(py_value_ptr));
+    }
+    else if(PyUnicode_Check(py_value_ptr))
+    {
+        PyObject* py_bytes_value_ptr = PyUnicode_AsLatin1String(py_value_ptr);
+        result.push_back(PyBytes_AS_STRING(py_bytes_value_ptr));
+        Py_DECREF(py_bytes_value_ptr);
     }
     else
     {
@@ -83,10 +139,16 @@ void convert2array(const object &py_value, Tango::DevVarStringArray & result)
         raise_(PyExc_TypeError, param_must_be_seq);
     }
     
-    if (PyString_Check(py_value_ptr))
+    if (PyBytes_Check(py_value_ptr))
     {
         result.length(1);
-        result[0] = CORBA::string_dup(PyString_AsString(py_value_ptr));
+        result[0] = CORBA::string_dup(PyBytes_AS_STRING(py_value_ptr));
+    }
+    else if(PyUnicode_Check(py_value_ptr))
+    {
+        PyObject* py_bytes_value_ptr = PyUnicode_AsLatin1String(py_value_ptr);
+        result[0] = CORBA::string_dup(PyBytes_AS_STRING(py_bytes_value_ptr));
+        Py_DECREF(py_bytes_value_ptr);
     }
     else
     {
@@ -140,35 +202,35 @@ void convert2array(const boost::python::object &py_value, Tango::DevVarLongStrin
     convert2array(py_str, result.svalue);
 }
 
-void from_py_object(object &py_obj, Tango::AttributeAlarm &attr_alarm)
+void from_py_object(bopy::object &py_obj, Tango::AttributeAlarm &attr_alarm)
 {
-    attr_alarm.min_alarm = extract<const char *>(py_obj.attr("min_alarm"));
-    attr_alarm.max_alarm = extract<const char *>(py_obj.attr("max_alarm"));
-    attr_alarm.min_warning = extract<const char *>(py_obj.attr("min_warning"));
-    attr_alarm.max_warning = extract<const char *>(py_obj.attr("max_warning"));
-    attr_alarm.delta_t = extract<const char *>(py_obj.attr("delta_t"));
-    attr_alarm.delta_val = extract<const char *>(py_obj.attr("delta_val"));
+    attr_alarm.min_alarm = obj_to_new_char(py_obj.attr("min_alarm"));
+    attr_alarm.max_alarm = obj_to_new_char(py_obj.attr("max_alarm"));
+    attr_alarm.min_warning = obj_to_new_char(py_obj.attr("min_warning"));
+    attr_alarm.max_warning = obj_to_new_char(py_obj.attr("max_warning"));
+    attr_alarm.delta_t = obj_to_new_char(py_obj.attr("delta_t"));
+    attr_alarm.delta_val = obj_to_new_char(py_obj.attr("delta_val"));
     convert2array(py_obj.attr("extensions"), attr_alarm.extensions);
 }
 
 void from_py_object(object &py_obj, Tango::ChangeEventProp &change_evt_prop)
 {
-    change_evt_prop.rel_change = extract<const char *>(py_obj.attr("rel_change"));
-    change_evt_prop.abs_change = extract<const char *>(py_obj.attr("abs_change"));
+    change_evt_prop.rel_change = obj_to_new_char(py_obj.attr("rel_change"));
+    change_evt_prop.abs_change = obj_to_new_char(py_obj.attr("abs_change"));
     convert2array(py_obj.attr("extensions"), change_evt_prop.extensions);
 }
 
 void from_py_object(object &py_obj, Tango::PeriodicEventProp &periodic_evt_prop)
 {
-    periodic_evt_prop.period = extract<const char *>(py_obj.attr("period"));
+    periodic_evt_prop.period = obj_to_new_char(py_obj.attr("period"));
     convert2array(py_obj.attr("extensions"), periodic_evt_prop.extensions);
 }
 
 void from_py_object(object &py_obj, Tango::ArchiveEventProp &archive_evt_prop)
 {
-    archive_evt_prop.rel_change = extract<const char *>(py_obj.attr("rel_change"));
-    archive_evt_prop.abs_change = extract<const char *>(py_obj.attr("abs_change"));
-    archive_evt_prop.period = extract<const char *>(py_obj.attr("period"));
+    archive_evt_prop.rel_change = obj_to_new_char(py_obj.attr("rel_change"));
+    archive_evt_prop.abs_change = obj_to_new_char(py_obj.attr("abs_change"));
+    archive_evt_prop.period = obj_to_new_char(py_obj.attr("period"));
     convert2array(py_obj.attr("extensions"), archive_evt_prop.extensions);
 }
 
@@ -186,66 +248,66 @@ void from_py_object(object &py_obj, Tango::EventProperties &evt_props)
 
 void from_py_object(object &py_obj, Tango::AttributeConfig &attr_conf)
 {
-    attr_conf.name = extract<const char *>(py_obj.attr("name"));
+    attr_conf.name = obj_to_new_char(py_obj.attr("name"));
     attr_conf.writable = extract<Tango::AttrWriteType>(py_obj.attr("writable"));
     attr_conf.data_format = extract<Tango::AttrDataFormat>(py_obj.attr("data_format"));
     attr_conf.data_type = extract<CORBA::Long>(py_obj.attr("data_type"));
     attr_conf.max_dim_x = extract<CORBA::Long>(py_obj.attr("max_dim_x"));
     attr_conf.max_dim_y = extract<CORBA::Long>(py_obj.attr("max_dim_y"));
-    attr_conf.description = extract<const char *>(py_obj.attr("description"));
-    attr_conf.label = extract<const char *>(py_obj.attr("label"));
-    attr_conf.unit = extract<const char *>(py_obj.attr("unit"));
-    attr_conf.standard_unit = extract<const char *>(py_obj.attr("standard_unit"));
-    attr_conf.display_unit = extract<const char *>(py_obj.attr("display_unit"));
-    attr_conf.format = extract<const char *>(py_obj.attr("format"));
-    attr_conf.min_value = extract<const char *>(py_obj.attr("min_value"));
-    attr_conf.max_value = extract<const char *>(py_obj.attr("max_value"));
-    attr_conf.min_alarm = extract<const char *>(py_obj.attr("min_alarm"));
-    attr_conf.max_alarm = extract<const char *>(py_obj.attr("max_alarm"));
-    attr_conf.writable_attr_name = extract<const char *>(py_obj.attr("writable_attr_name"));
+    attr_conf.description = obj_to_new_char(py_obj.attr("description"));
+    attr_conf.label = obj_to_new_char(py_obj.attr("label"));
+    attr_conf.unit = obj_to_new_char(py_obj.attr("unit"));
+    attr_conf.standard_unit = obj_to_new_char(py_obj.attr("standard_unit"));
+    attr_conf.display_unit = obj_to_new_char(py_obj.attr("display_unit"));
+    attr_conf.format = obj_to_new_char(py_obj.attr("format"));
+    attr_conf.min_value = obj_to_new_char(py_obj.attr("min_value"));
+    attr_conf.max_value = obj_to_new_char(py_obj.attr("max_value"));
+    attr_conf.min_alarm = obj_to_new_char(py_obj.attr("min_alarm"));
+    attr_conf.max_alarm = obj_to_new_char(py_obj.attr("max_alarm"));
+    attr_conf.writable_attr_name = obj_to_new_char(py_obj.attr("writable_attr_name"));
     convert2array(py_obj.attr("extensions"), attr_conf.extensions);
 }
 
 void from_py_object(object &py_obj, Tango::AttributeConfig_2 &attr_conf)
 {
-    attr_conf.name = extract<const char *>(py_obj.attr("name"));
+    attr_conf.name = obj_to_new_char(py_obj.attr("name"));
     attr_conf.writable = extract<Tango::AttrWriteType>(py_obj.attr("writable"));
     attr_conf.data_format = extract<Tango::AttrDataFormat>(py_obj.attr("data_format"));
     attr_conf.data_type = extract<CORBA::Long>(py_obj.attr("data_type"));
     attr_conf.max_dim_x = extract<CORBA::Long>(py_obj.attr("max_dim_x"));
     attr_conf.max_dim_y = extract<CORBA::Long>(py_obj.attr("max_dim_y"));
-    attr_conf.description = extract<const char *>(py_obj.attr("description"));
-    attr_conf.label = extract<const char *>(py_obj.attr("label"));
-    attr_conf.unit = extract<const char *>(py_obj.attr("unit"));
-    attr_conf.standard_unit = extract<const char *>(py_obj.attr("standard_unit"));
-    attr_conf.display_unit = extract<const char *>(py_obj.attr("display_unit"));
-    attr_conf.format = extract<const char *>(py_obj.attr("format"));
-    attr_conf.min_value = extract<const char *>(py_obj.attr("min_value"));
-    attr_conf.max_value = extract<const char *>(py_obj.attr("max_value"));
-    attr_conf.min_alarm = extract<const char *>(py_obj.attr("min_alarm"));
-    attr_conf.max_alarm = extract<const char *>(py_obj.attr("max_alarm"));
-    attr_conf.writable_attr_name = extract<const char *>(py_obj.attr("writable_attr_name"));
+    attr_conf.description = obj_to_new_char(py_obj.attr("description"));
+    attr_conf.label = obj_to_new_char(py_obj.attr("label"));
+    attr_conf.unit = obj_to_new_char(py_obj.attr("unit"));
+    attr_conf.standard_unit = obj_to_new_char(py_obj.attr("standard_unit"));
+    attr_conf.display_unit = obj_to_new_char(py_obj.attr("display_unit"));
+    attr_conf.format = obj_to_new_char(py_obj.attr("format"));
+    attr_conf.min_value = obj_to_new_char(py_obj.attr("min_value"));
+    attr_conf.max_value = obj_to_new_char(py_obj.attr("max_value"));
+    attr_conf.min_alarm = obj_to_new_char(py_obj.attr("min_alarm"));
+    attr_conf.max_alarm = obj_to_new_char(py_obj.attr("max_alarm"));
+    attr_conf.writable_attr_name = obj_to_new_char(py_obj.attr("writable_attr_name"));
     attr_conf.level = extract<Tango::DispLevel>(py_obj.attr("level"));
     convert2array(py_obj.attr("extensions"), attr_conf.extensions);
 }
 
 void from_py_object(object &py_obj, Tango::AttributeConfig_3 &attr_conf)
 {
-    attr_conf.name = extract<const char *>(py_obj.attr("name"));
+    attr_conf.name = obj_to_new_char(py_obj.attr("name"));
     attr_conf.writable = extract<Tango::AttrWriteType>(py_obj.attr("writable"));
     attr_conf.data_format = extract<Tango::AttrDataFormat>(py_obj.attr("data_format"));
     attr_conf.data_type = extract<CORBA::Long>(py_obj.attr("data_type"));
     attr_conf.max_dim_x = extract<CORBA::Long>(py_obj.attr("max_dim_x"));
     attr_conf.max_dim_y = extract<CORBA::Long>(py_obj.attr("max_dim_y"));
-    attr_conf.description = extract<const char *>(py_obj.attr("description"));
-    attr_conf.label = extract<const char *>(py_obj.attr("label"));
-    attr_conf.unit = extract<const char *>(py_obj.attr("unit"));
-    attr_conf.standard_unit = extract<const char *>(py_obj.attr("standard_unit"));
-    attr_conf.display_unit = extract<const char *>(py_obj.attr("display_unit"));
-    attr_conf.format = extract<const char *>(py_obj.attr("format"));
-    attr_conf.min_value = extract<const char *>(py_obj.attr("min_value"));
-    attr_conf.max_value = extract<const char *>(py_obj.attr("max_value"));
-    attr_conf.writable_attr_name = extract<const char *>(py_obj.attr("writable_attr_name"));
+    attr_conf.description = obj_to_new_char(py_obj.attr("description"));
+    attr_conf.label = obj_to_new_char(py_obj.attr("label"));
+    attr_conf.unit = obj_to_new_char(py_obj.attr("unit"));
+    attr_conf.standard_unit = obj_to_new_char(py_obj.attr("standard_unit"));
+    attr_conf.display_unit = obj_to_new_char(py_obj.attr("display_unit"));
+    attr_conf.format = obj_to_new_char(py_obj.attr("format"));
+    attr_conf.min_value = obj_to_new_char(py_obj.attr("min_value"));
+    attr_conf.max_value = obj_to_new_char(py_obj.attr("max_value"));
+    attr_conf.writable_attr_name = obj_to_new_char(py_obj.attr("writable_attr_name"));
     attr_conf.level = extract<Tango::DispLevel>(py_obj.attr("level"));
     
     object py_att_alarm = py_obj.attr("att_alarm");
