@@ -21,42 +21,60 @@
    
 *******************************************************************************/
 
-#include <boost/python.hpp>
+#include "precompiled_header.hpp"
 #include <tango.h>
 #include "tango_numpy.h"
 #include "device_attribute.h"
 
 using namespace boost::python;
 
+const int i = 1;
+#define IS_BIGENDIAN() ( (*(char*)&i) == 0 )
+
 namespace PyEncodedAttribute
 {
+
     /// This callback is run to delete char* objects.
     /// It is called by python. The array was associated with an attribute
     /// value object that is not being used anymore.
     /// @param ptr_ The array object.
     /// @param type_ The type of data. We don't need it for now
-
-    static void __ptr_deleter(void * ptr_, void *type_)
-    {
-        long t = reinterpret_cast<long>(type_);
-        if (1 == t)
-            delete [] (static_cast<unsigned char*>(ptr_));
-        else if (2 == t)
-            delete [] (static_cast<unsigned short*>(ptr_));
-        else if (4 == t)
-            delete [] (static_cast<uint32_t*>(ptr_));
-    }
+#    ifdef PYCAPSULE_OLD
+         template<long type>
+         static void __ptr_deleter(void * ptr_)
+         {
+             if (1 == type)
+                 delete [] (static_cast<unsigned char*>(ptr_));
+             else if (2 == type)
+                 delete [] (static_cast<unsigned short*>(ptr_));
+             else if (4 == type)
+                 delete [] (static_cast<Tango::DevULong*>(ptr_));
+         }
+#    else
+         template<long type>
+         static void __ptr_deleter(PyObject* obj)
+         {
+             void * ptr_ = PyCapsule_GetPointer(obj, NULL);
+             if (1 == type)
+                 delete [] (static_cast<unsigned char*>(ptr_));
+             else if (2 == type)
+                 delete [] (static_cast<unsigned short*>(ptr_));
+             else if (4 == type)
+                 delete [] (static_cast<Tango::DevULong*>(ptr_));
+         }
+#    endif
     
     void encode_gray8(Tango::EncodedAttribute &self, object py_value, int w, int h)
     {
         PyObject *py_value_ptr = py_value.ptr();
         unsigned char *buffer = NULL;
-        if (PyString_Check(py_value_ptr))
+        if (PyBytes_Check(py_value_ptr))
         {
-            buffer = reinterpret_cast<unsigned char*>(PyString_AsString(py_value_ptr));
+            buffer = reinterpret_cast<unsigned char*>(PyBytes_AsString(py_value_ptr));
             self.encode_gray8(buffer, w, h);
             return;
         }
+#ifndef DISABLE_PYTANGO_NUMPY
         else if (PyArray_Check(py_value_ptr))
         {
             w = PyArray_DIM(py_value_ptr, 1);
@@ -66,11 +84,14 @@ namespace PyEncodedAttribute
             self.encode_gray8(buffer, w, h);
             return;
         }
+#endif
         // It must be a py sequence
         // we are sure that w and h are given by python (see encoded_attribute.py)
-        unsigned char b[w*h];
-        buffer = b;
-        unsigned char *p = b;
+        const int length = w*h;
+        unsigned char *raw_b = new unsigned char[length];
+        unique_pointer<unsigned char> b(raw_b);
+        buffer = raw_b;
+        unsigned char *p = raw_b;
         int w_bytes = w;
         for (long y=0; y<h; ++y)
         {
@@ -85,16 +106,16 @@ namespace PyEncodedAttribute
                 boost::python::throw_error_already_set();
             }
             // The given object is a sequence of strings were each string is the entire row
-            if (PyString_Check(row))
+            if (PyBytes_Check(row))
             {
-                if (PyString_Size(row) != w_bytes)
+                if (PyBytes_Size(row) != w_bytes)
                 {
                     Py_DECREF(row);
                     PyErr_SetString(PyExc_TypeError,
                         "All sequences inside a sequence must have same size");
                     boost::python::throw_error_already_set();
                 }
-                memcpy(p, PyString_AsString(row), w_bytes);
+                memcpy(p, PyBytes_AsString(row), w_bytes);
                 p += w;
             }
             else
@@ -115,9 +136,9 @@ namespace PyEncodedAttribute
                         Py_DECREF(row);
                         boost::python::throw_error_already_set();
                     }
-                    if (PyString_Check(cell))
+                    if (PyBytes_Check(cell))
                     {
-                        if (PyString_Size(cell) != 1)
+                        if (PyBytes_Size(cell) != 1)
                         {
                             Py_DECREF(row);
                             Py_DECREF(cell);
@@ -125,13 +146,13 @@ namespace PyEncodedAttribute
                                 "All string items must have length one");
                             boost::python::throw_error_already_set();
                         }
-                        char byte = PyString_AsString(cell)[0];
+                        char byte = PyBytes_AsString(cell)[0];
                         *p = byte;
                     }
-                    else if (PyInt_Check(cell) || PyLong_Check(cell))
+                    else if (PyLong_Check(cell))
                     {
                         long byte = PyLong_AsLong(cell);
-                        if (byte==-1 and PyErr_Occurred())
+                        if (byte==-1 && PyErr_Occurred())
                         {
                             Py_DECREF(row);
                             Py_DECREF(cell);
@@ -157,31 +178,35 @@ namespace PyEncodedAttribute
         self.encode_gray8(buffer, w, h);
     }
 
-    void encode_gray16(Tango::EncodedAttribute &self, object py_value, int w, int h)
+    void encode_jpeg_gray8(Tango::EncodedAttribute &self, object py_value, int w, int h, double quality)
     {
         PyObject *py_value_ptr = py_value.ptr();
-        unsigned short *buffer = NULL;
-        if (PyString_Check(py_value_ptr))
+        unsigned char *buffer = NULL;
+        if (PyBytes_Check(py_value_ptr))
         {
-            buffer = reinterpret_cast<unsigned short*>(PyString_AsString(py_value_ptr));
-            self.encode_gray16(buffer, w, h);
+            buffer = reinterpret_cast<unsigned char*>(PyBytes_AsString(py_value_ptr));
+            self.encode_jpeg_gray8(buffer, w, h, quality);
             return;
         }
+#ifndef DISABLE_PYTANGO_NUMPY
         else if (PyArray_Check(py_value_ptr))
         {
             w = PyArray_DIM(py_value_ptr, 1);
             h = PyArray_DIM(py_value_ptr, 0);
             
-            buffer = static_cast<unsigned short*>(PyArray_DATA(py_value_ptr));
-            self.encode_gray16(buffer, w, h);
+            buffer = static_cast<unsigned char*>(PyArray_DATA(py_value_ptr));
+            self.encode_jpeg_gray8(buffer, w, h, quality);
             return;
         }
+#endif
         // It must be a py sequence
         // we are sure that w and h are given by python (see encoded_attribute.py)
-        unsigned short b[w*h];
-        buffer = b;
-        unsigned short *p = b;
-        int w_bytes = 2*w;
+        const int length = w*h;
+        unsigned char *raw_b = new unsigned char[length];
+        unique_pointer<unsigned char> b(raw_b);
+        buffer = raw_b;
+        unsigned char *p = raw_b;
+        int w_bytes = w;
         for (long y=0; y<h; ++y)
         {
             PyObject *row = PySequence_GetItem(py_value_ptr, y);
@@ -195,16 +220,16 @@ namespace PyEncodedAttribute
                 boost::python::throw_error_already_set();
             }
             // The given object is a sequence of strings were each string is the entire row
-            if (PyString_Check(row))
+            if (PyBytes_Check(row))
             {
-                if (PyString_Size(row) != w_bytes)
+                if (PyBytes_Size(row) != w_bytes)
                 {
                     Py_DECREF(row);
                     PyErr_SetString(PyExc_TypeError,
                         "All sequences inside a sequence must have same size");
                     boost::python::throw_error_already_set();
                 }
-                memcpy(p, PyString_AsString(row), w_bytes);
+                memcpy(p, PyBytes_AsString(row), w_bytes);
                 p += w;
             }
             else
@@ -225,9 +250,123 @@ namespace PyEncodedAttribute
                         Py_DECREF(row);
                         boost::python::throw_error_already_set();
                     }
-                    if (PyString_Check(cell))
+                    if (PyBytes_Check(cell))
                     {
-                        if (PyString_Size(cell) != 2)
+                        if (PyBytes_Size(cell) != 1)
+                        {
+                            Py_DECREF(row);
+                            Py_DECREF(cell);
+                            PyErr_SetString(PyExc_TypeError,
+                                "All string items must have length one");
+                            boost::python::throw_error_already_set();
+                        }
+                        char byte = PyBytes_AsString(cell)[0];
+                        *p = byte;
+                    }
+                    else if (PyLong_Check(cell))
+                    {
+                        long byte = PyLong_AsLong(cell);
+                        if (byte==-1 && PyErr_Occurred())
+                        {
+                            Py_DECREF(row);
+                            Py_DECREF(cell);
+                            boost::python::throw_error_already_set();
+                        }
+                        if (byte < 0 || byte > 255)
+                        {
+                            Py_DECREF(row);
+                            Py_DECREF(cell);
+                            PyErr_SetString(PyExc_TypeError,
+                                "int item not in range(256)");
+                            boost::python::throw_error_already_set();
+                        }
+                        *p = (unsigned char)byte;
+                        
+                    }
+                    Py_DECREF(cell);
+                    p++;
+                }
+            }
+            Py_DECREF(row);
+        }
+        self.encode_jpeg_gray8(buffer, w, h, quality);
+    }
+    
+    void encode_gray16(Tango::EncodedAttribute &self, object py_value, int w, int h)
+    {
+        PyObject *py_value_ptr = py_value.ptr();
+        unsigned short *buffer = NULL;
+        if (PyBytes_Check(py_value_ptr))
+        {
+            buffer = reinterpret_cast<unsigned short*>(PyBytes_AsString(py_value_ptr));
+            self.encode_gray16(buffer, w, h);
+            return;
+        }
+#ifndef DISABLE_PYTANGO_NUMPY
+        else if (PyArray_Check(py_value_ptr))
+        {
+            w = PyArray_DIM(py_value_ptr, 1);
+            h = PyArray_DIM(py_value_ptr, 0);
+            
+            buffer = static_cast<unsigned short*>(PyArray_DATA(py_value_ptr));
+            self.encode_gray16(buffer, w, h);
+            return;
+        }
+#endif
+        // It must be a py sequence
+        // we are sure that w and h are given by python (see encoded_attribute.py)
+        const int length = w*h;
+        unsigned short *raw_b = new unsigned short[length];
+        unique_pointer<unsigned short> b(raw_b);
+        buffer = raw_b;
+        unsigned short *p = raw_b;
+        int w_bytes = 2*w;
+        for (long y=0; y<h; ++y)
+        {
+            PyObject *row = PySequence_GetItem(py_value_ptr, y);
+            if (!row) boost::python::throw_error_already_set();
+            if (!PySequence_Check(row))
+            {
+                Py_DECREF(row);
+                PyErr_SetString(PyExc_TypeError,
+                    "Expected sequence (str, numpy.ndarray, list, tuple or "
+                    "bytearray) inside a sequence");
+                boost::python::throw_error_already_set();
+            }
+            // The given object is a sequence of strings were each string is the entire row
+            if (PyBytes_Check(row))
+            {
+                if (PyBytes_Size(row) != w_bytes)
+                {
+                    Py_DECREF(row);
+                    PyErr_SetString(PyExc_TypeError,
+                        "All sequences inside a sequence must have same size");
+                    boost::python::throw_error_already_set();
+                }
+                memcpy(p, PyBytes_AsString(row), w_bytes);
+                p += w;
+            }
+            else
+            {
+                if (PySequence_Size(row) != w)
+                {
+                    Py_DECREF(row);
+                    PyErr_SetString(PyExc_TypeError,
+                        "All sequences inside a sequence must have same size");
+                    boost::python::throw_error_already_set();
+                }
+                
+                for (long x=0; x<w; ++x)
+                {
+                    PyObject *cell = PySequence_GetItem(row, x);
+                    if (!cell)
+                    {
+                        Py_DECREF(row);
+                        boost::python::throw_error_already_set();
+                    }
+                    if (PyBytes_Check(cell))
+                    {
+                        if (PyBytes_Size(cell) != 2)
                         {
                             Py_DECREF(row);
                             Py_DECREF(cell);
@@ -235,12 +374,12 @@ namespace PyEncodedAttribute
                                 "All string items must have length two");
                             boost::python::throw_error_already_set();
                         }
-                        unsigned short *word = reinterpret_cast<unsigned short *>(PyString_AsString(cell));
+                        unsigned short *word = reinterpret_cast<unsigned short *>(PyBytes_AsString(cell));
                         *p = *word;
                     }
-                    else if (PyInt_Check(cell) || PyLong_Check(cell))
+                    else if (PyLong_Check(cell))
                     {
-                        unsigned short word = PyLong_AsUnsignedLong(cell);
+                        unsigned short word = (unsigned short)PyLong_AsUnsignedLong(cell);
                         if (PyErr_Occurred())
                         {
                             Py_DECREF(row);
@@ -265,28 +404,32 @@ namespace PyEncodedAttribute
         }
         self.encode_gray16(buffer, w, h);
     }
-    
+
     void encode_rgb24(Tango::EncodedAttribute &self, object py_value, int w, int h)
     {
         PyObject *py_value_ptr = py_value.ptr();
         unsigned char *buffer = NULL;
-        if (PyString_Check(py_value_ptr))
+        if (PyBytes_Check(py_value_ptr))
         {
-            buffer = reinterpret_cast<unsigned char*>(PyString_AsString(py_value_ptr));
+            buffer = reinterpret_cast<unsigned char*>(PyBytes_AsString(py_value_ptr));
             self.encode_rgb24(buffer, w, h);
             return;
         }
+#ifndef DISABLE_PYTANGO_NUMPY
         else if (PyArray_Check(py_value_ptr))
         {
             buffer = static_cast<unsigned char*>(PyArray_DATA(py_value_ptr));
             self.encode_rgb24(buffer, w, h);
             return;
         }
+#endif
         // It must be a py sequence
         // we are sure that w and h are given by python (see encoded_attribute.py)
-        unsigned char b[w*h];
-        buffer = b;
-        unsigned char *p = b;
+        const int length = w*h;
+        unsigned char *raw_b = new unsigned char[length];
+        unique_pointer<unsigned char> b(raw_b);
+        buffer = raw_b;
+        unsigned char *p = raw_b;
         int w_bytes = 3*w;
         for (long y=0; y<h; ++y)
         {
@@ -301,16 +444,16 @@ namespace PyEncodedAttribute
                 boost::python::throw_error_already_set();
             }
             // The given object is a sequence of strings were each string is the entire row
-            if (PyString_Check(row))
+            if (PyBytes_Check(row))
             {
-                if (PyString_Size(row) != w_bytes)
+                if (PyBytes_Size(row) != w_bytes)
                 {
                     Py_DECREF(row);
                     PyErr_SetString(PyExc_TypeError,
                         "All sequences inside a sequence must have same size");
                     boost::python::throw_error_already_set();
                 }
-                memcpy(p, PyString_AsString(row), w_bytes);
+                memcpy(p, PyBytes_AsString(row), w_bytes);
                 p += w;
             }
             else
@@ -331,9 +474,9 @@ namespace PyEncodedAttribute
                         Py_DECREF(row);
                         boost::python::throw_error_already_set();
                     }
-                    if (PyString_Check(cell))
+                    if (PyBytes_Check(cell))
                     {
-                        if (PyString_Size(cell) != 3)
+                        if (PyBytes_Size(cell) != 3)
                         {
                             Py_DECREF(row);
                             Py_DECREF(cell);
@@ -341,31 +484,31 @@ namespace PyEncodedAttribute
                                 "All string items must have length one");
                             boost::python::throw_error_already_set();
                         }
-                        char *byte = PyString_AsString(cell);
+                        char *byte = PyBytes_AsString(cell);
                         *p = *byte; p++; byte++;
                         *p = *byte; p++; byte++;
                         *p = *byte; p++;
                     }
-                    else if (PyInt_Check(cell) || PyLong_Check(cell))
+                    else if (PyLong_Check(cell))
                     {
                         long byte = PyLong_AsLong(cell);
-                        if (byte==-1 and PyErr_Occurred())
+                        if (byte==-1 && PyErr_Occurred())
                         {
                             Py_DECREF(row);
                             Py_DECREF(cell);
                             boost::python::throw_error_already_set();
                         }
-                        if (BYTE_ORDER == LITTLE_ENDIAN)
+                        if (IS_BIGENDIAN())
                         {
-                            *p = (byte) & 0xFF; p++;
-                            *p = (byte >>  8) & 0xFF; p++;
-                            *p = (byte >> 16) & 0xFF; p++;
+                            *p = (unsigned char)(byte >> 16) & 0xFF; p++;
+                            *p = (unsigned char)(byte >>  8) & 0xFF; p++;
+                            *p = (unsigned char)(byte) & 0xFF; p++;
                         }
                         else
                         {
-                            *p = (byte >> 16) & 0xFF; p++;
-                            *p = (byte >>  8) & 0xFF; p++;
-                            *p = (byte) & 0xFF; p++;
+                            *p = (unsigned char)(byte) & 0xFF; p++;
+                            *p = (unsigned char)(byte >>  8) & 0xFF; p++;
+                            *p = (unsigned char)(byte >> 16) & 0xFF; p++;
                         }
                     }
                     Py_DECREF(cell);
@@ -375,7 +518,238 @@ namespace PyEncodedAttribute
         }
         self.encode_rgb24(buffer, w, h);
     }
-    
+
+    void encode_jpeg_rgb24(Tango::EncodedAttribute &self, object py_value, int w, int h, double quality)
+    {
+        PyObject *py_value_ptr = py_value.ptr();
+        unsigned char *buffer = NULL;
+        if (PyBytes_Check(py_value_ptr))
+        {
+            buffer = reinterpret_cast<unsigned char*>(PyBytes_AsString(py_value_ptr));
+            self.encode_jpeg_rgb24(buffer, w, h, quality);
+            return;
+        }
+#ifndef DISABLE_PYTANGO_NUMPY
+        else if (PyArray_Check(py_value_ptr))
+        {
+            buffer = static_cast<unsigned char*>(PyArray_DATA(py_value_ptr));
+            self.encode_jpeg_rgb24(buffer, w, h, quality);
+            return;
+        }
+#endif
+        // It must be a py sequence
+        // we are sure that w and h are given by python (see encoded_attribute.py)
+		const int length = w*h;
+        unsigned char *raw_b = new unsigned char[length];
+        unique_pointer<unsigned char> b(raw_b);
+		buffer = raw_b;
+        unsigned char *p = raw_b;
+        int w_bytes = 3*w;
+        for (long y=0; y<h; ++y)
+        {
+            PyObject *row = PySequence_GetItem(py_value_ptr, y);
+            if (!row) boost::python::throw_error_already_set();
+            if (!PySequence_Check(row))
+            {
+                Py_DECREF(row);
+                PyErr_SetString(PyExc_TypeError,
+                    "Expected sequence (str, numpy.ndarray, list, tuple or "
+                    "bytearray) inside a sequence");
+                boost::python::throw_error_already_set();
+            }
+            // The given object is a sequence of strings were each string is the entire row
+            if (PyBytes_Check(row))
+            {
+                if (PyBytes_Size(row) != w_bytes)
+                {
+                    Py_DECREF(row);
+                    PyErr_SetString(PyExc_TypeError,
+                        "All sequences inside a sequence must have same size");
+                    boost::python::throw_error_already_set();
+                }
+                memcpy(p, PyBytes_AsString(row), w_bytes);
+                p += w;
+            }
+            else
+            {
+                if (PySequence_Size(row) != w)
+                {
+                    Py_DECREF(row);
+                    PyErr_SetString(PyExc_TypeError,
+                        "All sequences inside a sequence must have same size");
+                    boost::python::throw_error_already_set();
+                }
+                
+                for (long x=0; x<w; ++x)
+                {
+                    PyObject *cell = PySequence_GetItem(row, x);
+                    if (!cell)
+                    {
+                        Py_DECREF(row);
+                        boost::python::throw_error_already_set();
+                    }
+                    if (PyBytes_Check(cell))
+                    {
+                        if (PyBytes_Size(cell) != 3)
+                        {
+                            Py_DECREF(row);
+                            Py_DECREF(cell);
+                            PyErr_SetString(PyExc_TypeError,
+                                "All string items must have length one");
+                            boost::python::throw_error_already_set();
+                        }
+                        char *byte = PyBytes_AsString(cell);
+                        *p = *byte; p++; byte++;
+                        *p = *byte; p++; byte++;
+                        *p = *byte; p++;
+                    }
+                    else if (PyLong_Check(cell))
+                    {
+                        long byte = PyLong_AsLong(cell);
+                        if (byte==-1 && PyErr_Occurred())
+                        {
+                            Py_DECREF(row);
+                            Py_DECREF(cell);
+                            boost::python::throw_error_already_set();
+                        }
+                        if (IS_BIGENDIAN())
+                        {
+                            *p = (unsigned char)(byte >> 16) & 0xFF; p++;
+                            *p = (unsigned char)(byte >>  8) & 0xFF; p++;
+                            *p = (unsigned char)(byte) & 0xFF; p++;
+                        }
+                        else
+                        {
+                            *p = (unsigned char)(byte) & 0xFF; p++;
+                            *p = (unsigned char)(byte >>  8) & 0xFF; p++;
+                            *p = (unsigned char)(byte >> 16) & 0xFF; p++;
+                        }
+                    }
+                    Py_DECREF(cell);
+                }
+            }
+            Py_DECREF(row);
+        }
+        self.encode_jpeg_rgb24(buffer, w, h, quality);
+    }
+
+    void encode_jpeg_rgb32(Tango::EncodedAttribute &self, object py_value, int w, int h, double quality)
+    {
+        PyObject *py_value_ptr = py_value.ptr();
+        unsigned char *buffer = NULL;
+        if (PyBytes_Check(py_value_ptr))
+        {
+            buffer = reinterpret_cast<unsigned char*>(PyBytes_AsString(py_value_ptr));
+            self.encode_jpeg_rgb32(buffer, w, h, quality);
+            return;
+        }
+#ifndef DISABLE_PYTANGO_NUMPY
+        else if (PyArray_Check(py_value_ptr))
+        {
+            buffer = static_cast<unsigned char*>(PyArray_DATA(py_value_ptr));
+            self.encode_jpeg_rgb32(buffer, w, h, quality);
+            return;
+        }
+#endif
+        // It must be a py sequence
+        // we are sure that w and h are given by python (see encoded_attribute.py)
+		const int length = w*h;
+        unsigned char *raw_b = new unsigned char[length];
+        unique_pointer<unsigned char> b(raw_b);
+		buffer = raw_b;
+        unsigned char *p = raw_b;
+        int w_bytes = 4*w;
+        for (long y=0; y<h; ++y)
+        {
+            PyObject *row = PySequence_GetItem(py_value_ptr, y);
+            if (!row) boost::python::throw_error_already_set();
+            if (!PySequence_Check(row))
+            {
+                Py_DECREF(row);
+                PyErr_SetString(PyExc_TypeError,
+                    "Expected sequence (str, numpy.ndarray, list, tuple or "
+                    "bytearray) inside a sequence");
+                boost::python::throw_error_already_set();
+            }
+            // The given object is a sequence of strings were each string is the entire row
+            if (PyBytes_Check(row))
+            {
+                if (PyBytes_Size(row) != w_bytes)
+                {
+                    Py_DECREF(row);
+                    PyErr_SetString(PyExc_TypeError,
+                        "All sequences inside a sequence must have same size");
+                    boost::python::throw_error_already_set();
+                }
+                memcpy(p, PyBytes_AsString(row), w_bytes);
+                p += w;
+            }
+            else
+            {
+                if (PySequence_Size(row) != w)
+                {
+                    Py_DECREF(row);
+                    PyErr_SetString(PyExc_TypeError,
+                        "All sequences inside a sequence must have same size");
+                    boost::python::throw_error_already_set();
+                }
+                
+                for (long x=0; x<w; ++x)
+                {
+                    PyObject *cell = PySequence_GetItem(row, x);
+                    if (!cell)
+                    {
+                        Py_DECREF(row);
+                        boost::python::throw_error_already_set();
+                    }
+                    if (PyBytes_Check(cell))
+                    {
+                        if (PyBytes_Size(cell) != 3)
+                        {
+                            Py_DECREF(row);
+                            Py_DECREF(cell);
+                            PyErr_SetString(PyExc_TypeError,
+                                "All string items must have length one");
+                            boost::python::throw_error_already_set();
+                        }
+                        char *byte = PyBytes_AsString(cell);
+                        *p = *byte; p++; byte++;
+                        *p = *byte; p++; byte++;
+                        *p = *byte; p++; byte++;
+                        *p = *byte; p++;
+                    }
+                    else if (PyLong_Check(cell))
+                    {
+                        long byte = PyLong_AsLong(cell);
+                        if (byte==-1 && PyErr_Occurred())
+                        {
+                            Py_DECREF(row);
+                            Py_DECREF(cell);
+                            boost::python::throw_error_already_set();
+                        }
+                        if (IS_BIGENDIAN())
+                        {
+                            *p = (unsigned char)(byte >> 24) & 0xFF; p++;
+                            *p = (unsigned char)(byte >> 16) & 0xFF; p++;
+                            *p = (unsigned char)(byte >>  8) & 0xFF; p++;
+                            *p = (unsigned char)(byte) & 0xFF; p++;
+                        }
+                        else
+                        {
+                            *p = (unsigned char)(byte) & 0xFF; p++;
+                            *p = (unsigned char)(byte >>  8) & 0xFF; p++;
+                            *p = (unsigned char)(byte >> 16) & 0xFF; p++;
+                            *p = (unsigned char)(byte >> 24) & 0xFF; p++;
+                        }
+                    }
+                    Py_DECREF(cell);
+                }
+            }
+            Py_DECREF(row);
+        }
+        self.encode_jpeg_rgb32(buffer, w, h, quality);
+    }
+
     PyObject *decode_gray8(Tango::EncodedAttribute &self, Tango::DeviceAttribute *attr, PyTango::ExtractAs extract_as)
     {
         unsigned char *buffer;
@@ -403,10 +777,10 @@ namespace PyEncodedAttribute
                 // PyCObject is intended for that kind of things. It's seen as a
                 // black box object from python. We assign him a function to be called
                 // when it is deleted -> the function deletes de data.
-                PyObject* guard = PyCObject_FromVoidPtrAndDesc(
-                    static_cast<void*>(ch_ptr),
-                    reinterpret_cast<void*>(1),
-                    __ptr_deleter);
+                PyObject* guard = PyCapsule_New(
+                        static_cast<void*>(ch_ptr),
+                        NULL,
+                        __ptr_deleter<1>);
                     
                 if (!guard)
                 {
@@ -428,7 +802,7 @@ namespace PyEncodedAttribute
                     throw_error_already_set();
                 }
                 size_t nb_bytes = width*height*sizeof(char);
-                PyObject *buffer_str = PyString_FromStringAndSize(ch_ptr, nb_bytes);
+                PyObject *buffer_str = PyBytes_FromStringAndSize(ch_ptr, nb_bytes);
                 if (!buffer_str)
                 {
                     Py_XDECREF(ret);
@@ -462,7 +836,7 @@ namespace PyEncodedAttribute
                     }
                     for (long x=0; x < width; ++x)
                     {
-                        PyTuple_SetItem(row, x, PyString_FromStringAndSize(ch_ptr + y*width+x, 1));
+                        PyTuple_SetItem(row, x, PyBytes_FromStringAndSize(ch_ptr + y*width+x, 1));
                     }
                     PyTuple_SetItem(ret, y, row);
                 }
@@ -470,7 +844,6 @@ namespace PyEncodedAttribute
                 delete [] buffer;
                 break;
             }
-            case PyTango::ExtractAsPyTango3:
             case PyTango::ExtractAsList:
             {
                 ret = PyList_New(height);
@@ -491,7 +864,7 @@ namespace PyEncodedAttribute
                     }
                     for (long x=0; x < width; ++x)
                     {
-                        PyList_SetItem(row, x, PyString_FromStringAndSize(ch_ptr + y*width+x, 1));
+                        PyList_SetItem(row, x, PyBytes_FromStringAndSize(ch_ptr + y*width+x, 1));
                     }
                     PyList_SetItem(ret, y, row);
                 }
@@ -523,6 +896,7 @@ namespace PyEncodedAttribute
         switch (extract_as)
         {
             case PyTango::ExtractAsNumpy:
+#ifndef DISABLE_PYTANGO_NUMPY
             {
                 npy_intp dims[2] = { height, width };
                 ret = PyArray_SimpleNewFromData(2, dims, NPY_USHORT, ch_ptr);
@@ -537,10 +911,10 @@ namespace PyEncodedAttribute
                 // PyCObject is intended for that kind of things. It's seen as a
                 // black box object from python. We assign him a function to be called
                 // when it is deleted -> the function deletes de data.
-                PyObject* guard = PyCObject_FromVoidPtrAndDesc(
+                PyObject* guard = PyCapsule_New(
                     static_cast<void*>(ch_ptr),
-                    reinterpret_cast<void*>(2),
-                    __ptr_deleter);
+                    NULL,
+                    __ptr_deleter<2>);
                     
                 if (!guard)
                 {
@@ -552,6 +926,7 @@ namespace PyEncodedAttribute
                 PyArray_BASE(ret) = guard;
                 break;
             }
+#endif
             case PyTango::ExtractAsString:
             {
                 ret = PyTuple_New(3);
@@ -562,7 +937,7 @@ namespace PyEncodedAttribute
                 }
                 size_t nb_bytes = width*height*sizeof(unsigned short);
                 
-                PyObject *buffer_str = PyString_FromStringAndSize(
+                PyObject *buffer_str = PyBytes_FromStringAndSize(
                     reinterpret_cast<char *>(ch_ptr), nb_bytes);
                 delete [] buffer;
                 
@@ -606,7 +981,6 @@ namespace PyEncodedAttribute
                 delete [] buffer;
                 break;
             }
-            case PyTango::ExtractAsPyTango3:
             case PyTango::ExtractAsList:
             {
                 ret = PyList_New(height);
@@ -659,6 +1033,7 @@ namespace PyEncodedAttribute
         switch (extract_as)
         {
             case PyTango::ExtractAsNumpy:
+#ifndef DISABLE_PYTANGO_NUMPY
             {
                 npy_intp dims[2] = { height, width };
                 ret = PyArray_SimpleNewFromData(2, dims, NPY_UINT32, ch_ptr);
@@ -672,11 +1047,11 @@ namespace PyEncodedAttribute
                 // the last copy of numpy.ndarray() disappears.
                 // PyCObject is intended for that kind of things. It's seen as a
                 // black box object from python. We assign him a function to be called
-                PyObject* guard = PyCObject_FromVoidPtrAndDesc(
                 // when it is deleted -> the function deletes de data.
+                PyObject* guard = PyCapsule_New(
                     static_cast<void*>(ch_ptr),
-                    reinterpret_cast<void*>(2),
-                    __ptr_deleter);
+                    NULL,
+                    __ptr_deleter<4>);
                     
                 if (!guard)
                 {
@@ -688,6 +1063,7 @@ namespace PyEncodedAttribute
                 PyArray_BASE(ret) = guard;
                 break;
             }
+#endif
             case PyTango::ExtractAsString:
             {
                 ret = PyTuple_New(3);
@@ -698,7 +1074,7 @@ namespace PyEncodedAttribute
                 }
                 size_t nb_bytes = width*height*4;
                 
-                PyObject *buffer_str = PyString_FromStringAndSize(
+                PyObject *buffer_str = PyBytes_FromStringAndSize(
                     reinterpret_cast<char *>(ch_ptr), nb_bytes);
                 delete [] buffer;
                 
@@ -736,8 +1112,16 @@ namespace PyEncodedAttribute
                     {
                         long idx = 4*(y*width+x);
                         // data comes in in big endian format
-                        uint32_t data;
-                        if (BYTE_ORDER == LITTLE_ENDIAN)
+						Tango::DevULong data;
+                        if (IS_BIGENDIAN())
+                        {
+                            char *p = reinterpret_cast<char *>(&data);
+                            *p = ch_ptr[idx++]; ++p;
+                            *p = ch_ptr[idx++]; ++p;
+                            *p = ch_ptr[idx++]; ++p;
+                            *p = ch_ptr[idx];
+                        }
+                        else
                         {
                             idx +=3;
                             char *p = reinterpret_cast<char *>(&data);
@@ -746,14 +1130,6 @@ namespace PyEncodedAttribute
                             *p = ch_ptr[idx--]; ++p;
                             *p = ch_ptr[idx];
                         }
-                        else
-                        {
-                            char *p = reinterpret_cast<char *>(&data);
-                            *p = ch_ptr[idx++]; ++p;
-                            *p = ch_ptr[idx++]; ++p;
-                            *p = ch_ptr[idx++]; ++p;
-                            *p = ch_ptr[idx];
-                        }
                         PyTuple_SetItem(row, x, PyLong_FromUnsignedLong(data));
                     }
                     PyTuple_SetItem(ret, y, row);
@@ -762,7 +1138,6 @@ namespace PyEncodedAttribute
                 delete [] buffer;
                 break;
             }
-            case PyTango::ExtractAsPyTango3:
             case PyTango::ExtractAsList:
             {
                 ret = PyList_New(height);
@@ -785,8 +1160,16 @@ namespace PyEncodedAttribute
                     {
                         long idx = 4*(y*width+x);
                         // data comes in in big endian format
-                        uint32_t data;
-                        if (BYTE_ORDER == LITTLE_ENDIAN)
+						Tango::DevULong data;
+                        if (IS_BIGENDIAN())
+                        {
+                            char *p = reinterpret_cast<char *>(&data);
+                            *p = ch_ptr[idx++]; ++p;
+                            *p = ch_ptr[idx++]; ++p;
+                            *p = ch_ptr[idx++]; ++p;
+                            *p = ch_ptr[idx];
+                        }
+                        else
                         {
                             idx +=3;
                             char *p = reinterpret_cast<char *>(&data);
@@ -795,14 +1178,6 @@ namespace PyEncodedAttribute
                             *p = ch_ptr[idx--]; ++p;
                             *p = ch_ptr[idx];
                         }
-                        else
-                        {
-                            char *p = reinterpret_cast<char *>(&data);
-                            *p = ch_ptr[idx++]; ++p;
-                            *p = ch_ptr[idx++]; ++p;
-                            *p = ch_ptr[idx++]; ++p;
-                            *p = ch_ptr[idx];
-                        }
                         PyList_SetItem(row, x, PyLong_FromUnsignedLong(data));
                     }
                     PyList_SetItem(ret, y, row);
@@ -837,8 +1212,11 @@ void export_encoded_attribute()
         .def("_encode_gray8", &PyEncodedAttribute::encode_gray8)
         .def("_encode_gray16", &PyEncodedAttribute::encode_gray16)
         .def("_encode_rgb24", &PyEncodedAttribute::encode_rgb24)
+        .def("_encode_jpeg_gray8", &PyEncodedAttribute::encode_jpeg_gray8)
+        .def("_encode_jpeg_rgb24", &PyEncodedAttribute::encode_jpeg_rgb24)
+        .def("_encode_jpeg_rgb32", &PyEncodedAttribute::encode_jpeg_rgb32)
         .def("_decode_gray8", &PyEncodedAttribute::decode_gray8)
         .def("_decode_gray16", &PyEncodedAttribute::decode_gray16)
         .def("_decode_rgb32", &PyEncodedAttribute::decode_rgb32)
     ;
-}
\ No newline at end of file
+}
